<?php

namespace App\Console\Commands;

use Illuminate\Console\Command;
use Illuminate\Support\Facades\Log;

class DigestMailCron extends Command
{
    /**
     * The name and signature of the console command.
     *
     * @var string
     */
    protected $signature = 'digestmail:cron';

    /**
     * The console command description.
     *
     * @var string
     */
    protected $description = 'Send Digest Mail';

    /**
     * Create a new command instance.
     *
     * @return void
     */
    public function __construct()
    {
        parent::__construct();
    }

    /**
     * Execute the console command.
     *
     * @return int
     */
    public function handle()
    {
        $subscription = \App\Models\Subscription::where('subscription_status', 1)->pluck('subscription_id');
        $emailDate = \App\Models\EmailDate::select('e_date')->orderBy('e_id', 'DESC')->first()->toArray();
        $EDate = $emailDate['e_date'];

        $digestSettings = \App\Models\DigestSetting::select('day', 'time')->first()->toArray();
        $sendMailDate = (date("Y-m-d", strtotime('next ' . $digestSettings['day'], strtotime($EDate)))) . ' ' . $digestSettings['time'];

        if (strtotime($sendMailDate) == strtotime(date('Y-m-d H:i:00'))) {
            foreach ($subscription as $val) {
                $articleData = collect();
                $articleIds = [];
                $visibleLevel = config('constant.visible_article')[$val];
                foreach ($visibleLevel as $key => $level) {

                    $query = \App\Models\Article::select('article_name', 'publish_date', 'article_id');

                    if ($level == 2) {
                        $nonVisibleArticle = $this->getArticleAfterApplyVisiableDate();
                        if (!empty($nonVisibleArticle)) {
                            $query->whereNotIn('article_id', $nonVisibleArticle);
                        }
                    }

                    $query->where(function ($q) use ($level) {
                        $q->orWhereRaw("JSON_CONTAINS(subscription_level, '\"$level\"')");
                    });
                    if (!empty($articleIds)) {
                        $query->whereNotIn("article_id", $articleIds);
                    }
                    $result = $query->where('publish_date', '>=',  $EDate)->where('publish_date', '<=',  $sendMailDate)->get();

                    if (!empty($result)) {
                        $articleData = $articleData->merge($result);
                    }
                    if (!empty($result->pluck('article_id')->toArray())) {
                        $articleIds = array_merge($articleIds, $result->pluck('article_id')->toArray());
                    }
                }

                if ($articleData->count() > 0) {
                    $appUserArray = \App\Models\AppUser::select('first_name', 'last_name', 'subscription_id', 'email', 'app_user_id', 'company')
                        ->where('subscription_id', $val)->get()->toArray();

                    if (!empty($appUserArray)) {

                        $appUserChunk = array_chunk($appUserArray, 100);

                        foreach ($appUserChunk as $appUser) {
                            $details = [
                                'article' => $articleData,
                                'app_user' => $appUser
                            ];
                            dispatch(new \App\Jobs\DigestMail($details));
                        }
                    }
                }
            }
            \App\Models\EmailDate::create(['e_date' => date('Y-m-d H:i:s'), 'e_status' => 1]);
        }
    }

    public function getArticleAfterApplyVisiableDate()
    {
        return \App\Models\Article::select('article_id')->whereRaw("JSON_CONTAINS(subscription_level, '\"2\"')")
            ->whereRaw("DATE(now()) < DATE_ADD(publish_date, INTERVAL visiable_after_days DAY)")
            ->whereNotNull('visiable_after_days')->get()->pluck('article_id');
    }
}
