<?php

namespace App\Exceptions;

use Illuminate\Foundation\Exceptions\Handler as ExceptionHandler;
use Tymon\JWTAuth\Exceptions\TokenExpiredException;
use Tymon\JWTAuth\Exceptions\TokenInvalidException;
use Illuminate\Auth\AuthenticationException;
use Throwable;

class Handler extends ExceptionHandler
{
    /**
     * A list of the exception types that are not reported.
     *
     * @var array<int, class-string<Throwable>>
     */
    protected $dontReport = [
        //
    ];

    /**
     * A list of the inputs that are never flashed for validation exceptions.
     *
     * @var array<int, string>
     */
    protected $dontFlash = [
        'current_password',
        'password',
        'password_confirmation',
    ];

    /**
     * Register the exception handling callbacks for the application.
     *
     * @return void
     */
    public function register()
    {
        $this->reportable(function (Throwable $e) {
            //
        });
    }

    public function render($request, Throwable $exception)
    {
        if ($exception instanceof TokenExpiredException) {
            if ($request->expectsJson()) {
                $error = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $exception->getMessage()
                ];
                return response()->json($error);
            } else {
                return parent::render($request, $exception);
            }
        }

        if ($exception instanceof TokenInvalidException) {
            if ($request->expectsJson()) {
                $error = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $exception->getMessage()
                ];
                return response()->json($error);
            } else {
                return parent::render($request, $exception);
            }
        }

        if ($exception instanceof AuthenticationException) {
            if ($request->expectsJson()) {
                $error = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $exception->getMessage()
                ];
                return response()->json($error);
            } else {
                return parent::render($request, $exception);
            }
        }

        return parent::render($request, $exception);
    }
}
