<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use App\Models\Article;
use App\Models\Video;
use App\Models\FavoriteArticle;
use Illuminate\Validation\Rule;
use App\Models\NotificationHistory;

class ArticleController extends Controller
{

    /**
     * @OA\Post(
     *     path="/api/non-subscribe-user-article-list",
     *     operationId="nonSubscribeUserArticleList",
     *     summary="Non Subscribe User Article List",
     *     description="List of article for non Subscribe user",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 @OA\Property(property="page", type="number"),
     *                 @OA\Property(property="limit", type="number"),
     *                 @OA\Property(property="search_text", type="number"),
     *                 @OA\Property(property="search_category", type="staring"),
     *                 @OA\Property(property="search_media", type="staring"),
     *                 @OA\Property(property="search_interest", type="staring"),
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Article List",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function nonSubscribeUserArticleList(Request $request)
    {
        try {

            $page = $request->input('page') ?? 1;
            $limit = $request->input('limit') ?? 10;
            $searchText = $request->input('search_text') ?? '';
            $searchCategory = $request->input('search_category') ? explode(',', $request->input('search_category')) : [];
            $searchMedia = $request->input('search_media') ? explode(',', $request->input('search_media')) : [];
            $searchInterest = $request->input('search_interest') ? explode(',', $request->input('search_interest')) : [];

            $offset = $this->offset($page, $limit);

            $video = [];
            $videoDB = Video::first()->toArray();
            if (!empty($videoDB)) {
                $video = [
                    [
                        'video' =>  $videoDB['video1'],
                        'content' => $videoDB['content1'],
                    ],
                    [
                        'video' => $videoDB['video2'],
                        'content' => $videoDB['content2'],
                    ],
                ];
            }

            $articleData = collect();
            $articleIds = [];

            foreach (config('constant.non_subscribe_user') as $key => $val) {

                $query = Article::select('article_id', 'article_name', 'search_text', 'image', 'media_selection', 'artical_short_desc', 'youtube_url', 'audio', 'resource', 'publisher', 'publish_date', 'thumbnail_image', 'subscription_level');
                if ($searchText != '') {
                    $searchTestArray = explode(' ', $searchText);
                    $query->where(function ($q) use ($searchTestArray) {
                        foreach ($searchTestArray as $key => $val) {
                            $q->orWhere('search_text', 'LIKE', '%' . $val . '%');
                        }
                    });
                }

                if (!empty($searchCategory)) {
                    $query->where(function ($q) use ($searchCategory) {
                        foreach ($searchCategory as $category) {
                            $q->orWhereRaw("JSON_CONTAINS(category_id, '\"$category\"')");
                        }
                    });
                }

                if (!empty($searchMedia)) {
                    $query->where(function ($q) use ($searchMedia) {
                        foreach ($searchMedia as $media) {
                            $q->orWhere('media_selection', $media);
                        }
                    });
                }

                if (!empty($searchInterest)) {
                    $query->where(function ($q) use ($searchInterest) {
                        foreach ($searchInterest as $interest) {
                            $q->orWhereRaw("JSON_CONTAINS(interest_id, '\"$interest\"')");
                        }
                    });
                }

                if (!empty($articleIds)) {
                    $query->whereNotIn("article_id", $articleIds);
                }

                $query->where(function ($q) use ($val) {
                    $q->orWhereRaw("JSON_CONTAINS(subscription_level, '\"$val\"')");
                });

                if ($val == 2) {
                    $nonVisibleArticle = $this->getArticleAfterApplyVisiableDate();
                    if (!empty($nonVisibleArticle)) {
                        $query->whereNotIn('article_id', $nonVisibleArticle);
                    }
                }

                $article =  $query->orderby('publish_date', 'DESC')->get();
                if (!empty($article)) {
                    $articleData = $articleData->merge($article);
                }
                if (!empty($article->pluck('article_id')->toArray())) {
                    $articleIds = array_merge($articleIds, $article->pluck('article_id')->toArray());
                }
            }

            $data = $this->paginate($articleData, $limit, $page);
            $article = $data->toArray()['data'];


            foreach ($article as $key => $val) {
                $images = [];
                if (!empty($val['image'])) {
                    foreach (json_decode($val['image'], true) as $image) {
                        if ($image != '') {
                            $images[]['imageurl'] = url('/') . '/image/' . $image;
                        }
                    }
                }
                $article[$key]['image'] = $images;

                if (!empty($val['resource'])) {
                    $article[$key]['resource'] = url('/') . '/resource/' . $val['resource'];
                }
                if (!empty($val['thumbnail_image'])) {
                    $article[$key]['thumbnail_image'] = url('/') . '/thumbnail_image/' . $val['thumbnail_image'];
                }
            }

            if (!empty($article) || !empty($video)) {
                $response = [
                    'data' => [
                        'video' => $video,
                        'article' => $article
                    ],
                    'status' => true,
                    'message' => __('messages.list', ['name' => 'Article']),
                    'total_page' => $data->lastPage()
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }

            $response['app_user_status'] = \Auth::guard('api')->user()->app_user_status;
            $response['subscription_start_date'] = \Auth::guard('api')->user()->subscription_start_date;
            $response['subscription_end_date'] = \Auth::guard('api')->user()->subscription_end_date;

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Post(
     *     path="/api/change-favorite",
     *     operationId="changeFavorite",
     *     summary="Add/Remove Favorite",
     *     description="Add/Remove Article in Favorite List",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={"article_id", "app_user_id", "is_favorite"},
     *                 @OA\Property(property="article_id", type="number"),
     *                 @OA\Property(property="app_user_id", type="number"),
     *                 @OA\Property(property="is_favorite", type="enum(1,0)")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Favorite",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function changeFavorite(Request $request)
    {

        try {

            $validator = Validator::make($request->all(), [
                'article_id' => 'required',
                'app_user_id' => 'required',
                'is_favorite' => 'required|in:1,0',
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $articleId = $request->get('article_id');
            $appUserID = $request->get('app_user_id');
            $isFavorite = $request->get('is_favorite');

            if ($isFavorite == 0) {
                FavoriteArticle::where(['article_id' =>  $articleId, 'app_user_id' =>  $appUserID])->delete();
            } else {
                FavoriteArticle::firstOrCreate(['article_id' =>  $articleId, 'app_user_id' =>  $appUserID]);
            }

            $response = [
                'data' => (object) [],
                'status' => true,
                'message' => 'Favorite'
            ];

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Post(
     *     path="/api/article-detail",
     *     operationId="articleDetail",
     *     summary="Article Detail",
     *     description="Article Detail",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={"article_id"},
     *                 @OA\Property(property="article_id", type="number"),
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Article Detail",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function articleDetail(Request $request)
    {

        try {

            $validator = Validator::make($request->all(), [
                'article_id' => [
                    'required',
                    Rule::exists('articles')->where(function ($query) {
                        $query->whereNull('deleted_at');
                    }),
                ],
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $articleId = $request->get('article_id');

            $article = Article::select('article_id', 'article_name',  'image', 'media_selection', 'artical_short_desc', 'youtube_url', 'audio', 'resource', 'publisher', 'publish_date', 'artical_long_desc', 'thumbnail_image')
                ->where('article_id', $articleId)->first()->toArray();

            if (!empty($article['image'])) {
                $images = [];
                foreach (json_decode($article['image'], true) as $image) {
                    if ($image != '') {
                        $images[]['imageurl'] = url('/') . '/image/' . $image;
                    }
                }
                $article['image'] = $images;
            }

            if (!empty($article['resource'])) {
                $article['resource'] = url('/') . '/resource/' . $article['resource'];
            }

            if (!empty($article['thumbnail_image'])) {
                $article['thumbnail_image'] = url('/') . '/thumbnail_image/' . $article['thumbnail_image'];
            }

            if (!empty($article)) {
                $response = [
                    'data' => $article,
                    'status' => true,
                    'message' => __('messages.detail', ['name' => 'Article'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }


            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Post(
     *     path="/api/subscribe-user-article-list",
     *     operationId="subscribeUserArticleList",
     *     summary="Subscribe User Article List",
     *     description="Subscribe User Article List",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 @OA\Property(property="page", type="number"),
     *                 @OA\Property(property="limit", type="number"),
     *                 @OA\Property(property="search_text", type="number"),
     *                 @OA\Property(property="search_category", type="staring"),
     *                 @OA\Property(property="search_media", type="staring"),
     *                 @OA\Property(property="search_interest", type="staring"),
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Article Detail",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function subscribeUserArticleList(Request $request)
    {
        try {

            $page = $request->input('page') ?? 1;
            $limit = $request->input('limit') ?? 10;
            $searchText = $request->input('search_text') ?? '';
            $searchCategory = $request->input('search_category') ? explode(',', $request->input('search_category')) : [];
            $searchMedia = $request->input('search_media') ? explode(',', $request->input('search_media')) : [];
            $searchInterest = $request->input('search_interest') ? explode(',', $request->input('search_interest')) : [];


            $offset = $this->offset($page, $limit);


            $subscriptionId = \Auth::guard('api')->user()->subscription_id;
            $companyId = \Auth::guard('api')->user()->company;

            if (!is_null($subscriptionId)) {
                $visibleLevel = config('constant.visible_article')[$subscriptionId];
            } else {
                $visibleLevel = config('constant.non_subscribe_user');
            }

            $articleData = collect();
            $articleIds = [];
            foreach ($visibleLevel as $key => $val) {

                $query = Article::select('article_id', 'article_name', 'search_text', 'image', 'media_selection', 'artical_short_desc', 'youtube_url', 'audio', 'resource', 'publisher', 'publish_date', 'thumbnail_image');
                if ($searchText != '') {
                    $searchTestArray = explode(' ', $searchText);
                    $query->where(function ($q) use ($searchTestArray) {
                        foreach ($searchTestArray as $key => $val) {
                            $q->orWhere('search_text', 'LIKE', '%' . $val . '%');
                        }
                    });
                }

                if (!empty($searchCategory)) {
                    $query->where(function ($q) use ($searchCategory) {
                        foreach ($searchCategory as $category) {
                            $q->orWhereRaw("JSON_CONTAINS(category_id, '\"$category\"')");
                        }
                    });
                }

                if (!empty($searchMedia)) {
                    $query->where(function ($q) use ($searchMedia) {
                        foreach ($searchMedia as $media) {
                            $q->orWhere('media_selection', $media);
                        }
                    });
                }

                if (!empty($searchInterest)) {
                    $query->where(function ($q) use ($searchInterest) {
                        foreach ($searchInterest as $interest) {
                            $q->orWhereRaw("JSON_CONTAINS(interest_id, '\"$interest\"')");
                        }
                    });
                }

                if ($val == 2) {
                    $nonVisibleArticle = $this->getArticleAfterApplyVisiableDate();
                    if (!empty($nonVisibleArticle)) {
                        $query->whereNotIn('article_id', $nonVisibleArticle);
                    }
                }

                $query->where(function ($q) use ($val) {
                    $q->orWhereRaw("JSON_CONTAINS(subscription_level, '\"$val\"')");
                });

                if (!empty($articleIds)) {
                    $query->whereNotIn("article_id", $articleIds);
                }

                if ($subscriptionId  == 6) {
                    $query->whereRaw("JSON_CONTAINS(company_id, '\"$companyId\"')");
                }

                $article =  $query->orderby('publish_date', 'DESC')->get();
                if (!empty($article)) {
                    $articleData = $articleData->merge($article);
                }
                if (!empty($article->pluck('article_id')->toArray())) {
                    $articleIds = array_merge($articleIds, $article->pluck('article_id')->toArray());
                }
            }

            $data = $this->paginate($articleData, $limit, $page);
            $article = $data->toArray()['data'];

            foreach ($article as $key => $val) {
                $images = [];
                if (!empty($val['image'])) {

                    foreach (json_decode($val['image'], true) as $image) {
                        if ($image != '') {
                            $images[]['imageurl'] = url('/') . '/image/' . $image;
                        }
                    }
                }
                $article[$key]['image'] = $images;

                if (!empty($val['resource'])) {
                    $article[$key]['resource'] = url('/') . '/resource/' . $val['resource'];
                }
                if (!empty($val['thumbnail_image'])) {
                    $article[$key]['thumbnail_image'] = url('/') . '/thumbnail_image/' . $val['thumbnail_image'];
                }
            }

            if (!empty($article) || !empty($video)) {
                $response = [
                    'data' => $article,
                    'total_page' =>  $data->lastPage(),
                    'status' => true,
                    'message' => __('messages.list', ['name' => 'Article'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }
            $response['subscription_expired'] = \Auth::guard('api')->user()->subscription_expired;
            $response['app_user_status'] = \Auth::guard('api')->user()->app_user_status;
            $response['subscription_start_date'] = \Auth::guard('api')->user()->subscription_start_date;
            $response['subscription_end_date'] = \Auth::guard('api')->user()->subscription_end_date;

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Post(
     *     path="/api/favorite-article",
     *     operationId="favoriteArticle",
     *     summary="Favorite Article List",
     *     description="Favorite Article List",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 @OA\Property(property="page", type="number"),
     *                 @OA\Property(property="limit", type="number"),
     *                 @OA\Property(property="search_text", type="number"),
     *                 @OA\Property(property="search_category", type="staring"),
     *                 @OA\Property(property="search_media", type="staring"),
     *                 @OA\Property(property="search_interest", type="staring"),
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Favorite Article",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function favoriteArticle(Request $request)
    {
        try {

            $page = $request->input('page') ?? 1;
            $limit = $request->input('limit') ?? 10;
            $searchText = $request->input('search_text') ?? '';
            $searchCategory = $request->input('search_category') ? explode(',', $request->input('search_category')) : [];
            $searchMedia = $request->input('search_media') ? explode(',', $request->input('search_media')) : [];
            $searchInterest = $request->input('search_interest') ? explode(',', $request->input('search_interest')) : [];


            $offset = $this->offset($page, $limit);


            $subscriptionId = \Auth::guard('api')->user()->subscription_id;
            $companyId = \Auth::guard('api')->user()->company;

            if (!is_null($subscriptionId)) {
                $visibleLevel = config('constant.visible_article')[$subscriptionId];
            } else {
                $visibleLevel = config('constant.non_subscribe_user');
            }
            $articleData = collect();
            $articleIds = [];

            foreach ($visibleLevel as $key => $val) {

                $query = Article::select('articles.article_id', 'article_name', 'search_text', 'image', 'media_selection', 'artical_short_desc', 'youtube_url', 'audio', 'resource', 'publisher', 'publish_date', 'thumbnail_image');
                $query->join('favorite_articles', 'favorite_articles.article_id', '=', 'articles.article_id');
                if ($searchText != '') {
                    $searchTestArray = explode(' ', $searchText);
                    $query->where(function ($q) use ($searchTestArray) {
                        foreach ($searchTestArray as $key => $val) {
                            $q->orWhere('search_text', 'LIKE', '%' . $val . '%');
                        }
                    });
                }

                if (!empty($searchCategory)) {
                    $query->where(function ($q) use ($searchCategory) {
                        foreach ($searchCategory as $category) {
                            $q->orWhereRaw("JSON_CONTAINS(category_id, '\"$category\"')");
                        }
                    });
                }

                if (!empty($searchMedia)) {
                    $query->where(function ($q) use ($searchMedia) {
                        foreach ($searchMedia as $media) {
                            $q->orWhere('media_selection', $media);
                        }
                    });
                }

                if (!empty($searchInterest)) {
                    $query->where(function ($q) use ($searchInterest) {
                        foreach ($searchInterest as $interest) {
                            $q->orWhereRaw("JSON_CONTAINS(interest_id, '\"$interest\"')");
                        }
                    });
                }

                if ($val == 2) {
                    $nonVisibleArticle = $this->getArticleAfterApplyVisiableDate();
                    if (!empty($nonVisibleArticle)) {
                        $query->whereNotIn('articles.article_id', $nonVisibleArticle);
                    }
                }

                if (!empty($articleIds)) {
                    $query->whereNotIn("articles.article_id", $articleIds);
                }


                $query->where(function ($q) use ($val) {
                    $q->orWhereRaw("JSON_CONTAINS(subscription_level, '\"$val\"')");
                });

                if ($subscriptionId  == 6) {
                    $query->whereRaw("JSON_CONTAINS(company_id, '\"$companyId\"')");
                }

                $query->where('favorite_articles.app_user_id', \Auth::guard('api')->user()->app_user_id);

                $article =  $query->orderby('publish_date', 'DESC')->get();
                if (!empty($article)) {
                    $articleData = $articleData->merge($article);
                }
                if (!empty($article->pluck('article_id')->toArray())) {
                    $articleIds = array_merge($articleIds, $article->pluck('article_id')->toArray());
                }
            }

            $data = $this->paginate($articleData, $limit, $page);
            $article = $data->toArray()['data'];

            foreach ($article as $key => $val) {
                $images = [];
                if (!empty($val['image'])) {
                    foreach (json_decode($val['image'], true) as $image) {
                        $images[]['imageurl'] = url('/') . '/image/' . $image;
                    }
                }
                $article[$key]['image'] = $images;

                if (!empty($val['resource'])) {
                    $article[$key]['resource'] = url('/') . '/resource/' . $val['resource'];
                }
                
                if (!empty($val['thumbnail_image'])) {
                    $article[$key]['thumbnail_image'] = url('/') . '/thumbnail_image/' . $val['thumbnail_image'];
                }
            }

            if (!empty($article) || !empty($video)) {
                $response = [
                    'data' => $article,
                    'total_page' =>  $data->lastPage(),
                    'status' => true,
                    'message' => __('messages.list', ['name' => 'Favorite Article'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }
            $response['subscription_expired'] = \Auth::guard('api')->user()->subscription_expired;
            $response['app_user_status'] = \Auth::guard('api')->user()->app_user_status;
            $response['subscription_start_date'] = \Auth::guard('api')->user()->subscription_start_date;
            $response['subscription_end_date'] = \Auth::guard('api')->user()->subscription_end_date;

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Get(
     *     path="/api/article-notification-list",
     *     operationId="articleNotificationList",
     *     summary="Article Notification List",
     *     description="Article Notification List",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\Response(
     *         response=200,
     *         description="Article Notification List",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function articleNotificationList(Request $request)
    {
        try {

            $appUserId = \Auth::guard('api')->user()->app_user_id;

            $notificationHistories = NotificationHistory::with('artical')->where('app_user_id', $appUserId)->get();
            
             foreach ($notificationHistories as $key => $val) {
                $images = [];
                 if($val->artical->image != ''){
                    if (!empty($val->artical->image)) {
                        foreach (json_decode($val->artical->image, true) as $image) {
                            if ($image != '') {
                                $images[]['imageurl'] = url('/') . '/image/' . $image;
                            }
                        }
                    }
                 }
                
                $val->artical->image = $images;

                if (!empty($val->artical->resource)) {
                    $val->artical->resource = url('/') . '/resource/' . $val->artical->resource;
                }
                if (!empty($val->artical->thumbnail_image)) {
                    $val->artical->thumbnail_image = url('/') . '/thumbnail_image/' . $val->artical->thumbnail_image;
                }
            }
            
            if ($notificationHistories->count() > 0) {
                $response = [
                    'data' => $notificationHistories,
                    'status' => true,
                    'message' => __('messages.list', ['name' => 'Notification Histories'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Post(
     *     path="/api/delete-article-notification",
     *     operationId="deleteArticleNotification",
     *     summary="Delete Article Notification",
     *     description="Delete Article Notification",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={"article_id"},
     *                 @OA\Property(property="article_id", type="number"),
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Delete Article Notification",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function deleteArticleNotification(Request $request)
    {
        try {

            $validator = Validator::make($request->all(), [
                'article_id' => 'required'
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $data = $request->all();
            $appUserId = \Auth::guard('api')->user()->app_user_id;
            $articleId = explode(',', $data['article_id']);

            NotificationHistory::where('app_user_id', $appUserId)->whereIn('article_id', $articleId)->delete();

            $response = [
                'data' => (object) [],
                'status' => true,
                'message' => __('messages.delete', ['name' => 'Notification Histories'])
            ];

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }
}
