<?php

namespace App\Http\Controllers\API;

use DB;
use Auth;
use Mail;
use Carbon\Carbon;
use App\Models\User;
use App\Models\AppUser;
use App\Models\Company;
use App\Models\Interest;
use Illuminate\Support\Str;
use App\Models\Subscription;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\Validator;

class AuthController extends Controller
{

    /**
     * @OA\Post(
     *     path="/api/login",
     *     operationId="login",
     *     summary="Login",
     *     description="Login",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     * @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={"email","password"},
     *                 @OA\Property(property="email", type="staring", example="joshua.manning@gmail.com"),
     *                 @OA\Property(property="password", type="staring", example="123456"),
     *                 @OA\Property(property="device_type", type="staring"),
     *                 @OA\Property(property="device_token", type="staring")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Login Successfully",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function login(Request $request)
    {
        try {

            $validator = Validator::make($request->all(), [
                'email' => 'required|string|email',
                'password' => 'required|string',
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $credentials = $request->only('email', 'password');

            $token = Auth::guard('api')->attempt($credentials);

            if (!$token) {
                return response()->json([
                    'data' => (object) [],
                    'status' => false,
                    'message' => 'Email and password invalid.',
                ], 401);
            }

            $user = Auth::guard('api')->user();

            if($user->app_user_status == 0){

                auth()->guard('api')->logout();
                return response()->json([
                    'data' => (object) [],
                    'status' => false,
                    'message' => 'Your Account is inactive. Please contact to admin.',
                ], 401);
            }

            $companyDetail = \App\Models\Company::select('company_id', 'company_name')->where('company_id', Auth::guard('api')->user()->company)->first();
            $user['company'] = $companyDetail;
            $user['children'] = $user->children;

            $user['subscription'] = [];
            if (!is_null($user->subscription_id) || $user->subscription_id != '') {
                $subscription = Subscription::select('subscription_level', 'subscription_name', 'subscription_content', 'subscription_price_per_month', 'subscription_price_per_year', 'subscription_status')->where('subscription_id', $user->subscription_id)->first();
                $user['subscription'] = !is_null($subscription) ? $subscription->toArray() : [];
            }

            $user['subscription_id'] = !empty($user['subscription_id']) ? $user['subscription_id']  :  null;

            $data = $request->all();
            if (isset($data['device_type']) && isset($data['device_token']) && \App\Models\DeviceToken::where([
                'device_type' => $data['device_type'],
                'device_token' => $data['device_token']
            ])->exists() == false) {
            \App\Models\DeviceToken::insert(['app_user_id' => $user->app_user_id, 'device_type' => ($request->input('device_type') == 0 ) ? 1 : $request->input('device_type'), 'device_token' => $request->input('device_token')]);
            } else {
                \App\Models\DeviceToken::where(['device_type' => $request->input('device_type'), 'device_token' => $request->input('device_token')])->update(['app_user_id' => $user->app_user_id]);
            }

            return response()->json([
                'status' => true,
                'message' => 'Login Successfully',
                'user' => $user,
                'token' => $token
            ]);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }


    public function registerGet(){
        $industry = Interest::select('interest_id', 'interest_name', 'interest_status')->status(1)->orderBy('interest_name', 'ASC')->get();
        return view('auth.register',compact('industry'));
    }

    /**
     * @OA\Post(
     *     path="/api/register",
     *     operationId="register",
     *     summary="Register ",
     *     description="Register",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={"first_name","last_name","email","password","company","title","phone_no","mailing_address","app_user_status","country_code"},
     *                 @OA\Property(property="first_name", type="staring"),
     *                 @OA\Property(property="last_name", type="staring"),
     *                 @OA\Property(property="email", type="email"),
     *                 @OA\Property(property="password", type="staring"),
     *                 @OA\Property(property="company", type="staring"),
     *                 @OA\Property(property="title", type="staring"),
     *                 @OA\Property(property="country_code", type="staring"),
     *                 @OA\Property(property="phone_no", type="number"),
     *                 @OA\Property(property="mailing_address", type="staring"),
     *                 @OA\Property(property="assistant_name", type="staring"),
     *                 @OA\Property(property="assistant_email", type="email"),
     *                 @OA\Property(property="assistant_country_code", type="staring"),
     *                @OA\Property(property="assistant_phone_no", type="number"),
     *                 @OA\Property(property="app_user_status", type="enum(1,0)"),
     *                 @OA\Property(property="birthday", type="date"),
     *                 @OA\Property(property="personal_interests", type="strings"),
     *                 @OA\Property(property="business_interests", type="strings"),
     *                 @OA\Property(property="spouse_name", type="strings"),
     *                 @OA\Property(property="spouse_birthday", type="date"),
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Register Successfully",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function register(Request $request)
    {
        try {

            // Validate user input
            $validator = Validator::make($request->all(), [
                'first_name' => 'required',
                'last_name' => 'required',
                'email' => ['required', 'email', Rule::unique('app_users')->whereNull('deleted_at')],
                'password' => 'min:6',
                'company' => 'required',
                'title' => 'required',
                'phone_no' => ['required', 'numeric', 'digits:10' , Rule::unique('app_users')->whereNull('deleted_at')],
                'mailing_address' => 'required',
                'assistant_name' => 'nullable',
                'assistant_email' => 'nullable|email',
                'assistant_phone_no' => 'nullable|numeric|digits:10',
                'app_user_status' => 'required',
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $existsCompany = Company::whereRaw('LOWER(`company_name`) = ? ', [trim(strtolower($request->company))])->first();
            if (is_null($existsCompany)) {
                $company = Company::create([
                    'company_name' => $request->company,
                    'company_status' => 1
                ]);
                $companyId = $company->company_id;
            } else {
                $companyId = $existsCompany->company_id;
            }

            // Create the user
            $user = AppUser::create([
                'first_name' => $request->first_name,
                'last_name' => $request->last_name,
                'email' => $request->email,
                'password' => Hash::make($request->password),
                'company' => $companyId,
                'title' => $request->title,
                'country_code' => isset($request->country_code) ? $request->country_code : 'US',
                'phone_no' => $request->phone_no,
                'mailing_address' => $request->mailing_address,
                'assistant_name' => isset($request->assistant_name) ? $request->assistant_name : '',
                'assistant_email' => isset($request->assistant_email) ? $request->assistant_email : '',
                'assistant_country_code' => isset($request->assistant_country_code) ? $request->assistant_country_code : 'US',
                'assistant_phone_no' => isset($request->assistant_phone_no) ? $request->assistant_phone_no : '',
                'app_user_status' => $request->app_user_status,
                // 'assistant_phone_no' => isset($request->assistant_phone_no) ? $request->assistant_phone_no : '',
                'birthday' => isset($request->birthday) ? $request->birthday : NULL,
                'personal_interests' => isset($request->personal_interests) ? $request->personal_interests : NULL,
                'business_interests' => isset($request->business_interests) ? $request->business_interests : NULL,
                'spouse_name' => isset($request->spouse_name) ? $request->spouse_name : NULL,
                'spouse_birthday' => isset($request->spouse_birthday) ? $request->spouse_birthday : NULL,
            ]);

            if (isset($request->name_children) && isset($request->bith_children)) {
                $children = [];
                $val['app_user_id'] = $user->app_user_id;
                $val['name'] = $request->name_children;
                $val['birthday'] = $request->bith_children;
                $children[] = $val;
                \App\Models\AppUserChildDetail::insert($children);
            }

            if (isset($request->children)) {
                $children = [];
                foreach ($request->children as $key => $val) {
                    $val['app_user_id'] = $user->app_user_id;
                    $val['birthday'] = date('Y-m-d', strtotime($val['birthday']));
                    $children[] = $val;
                }
                \App\Models\AppUserChildDetail::insert($children);
            }

            return response()->json([
                'status' => true,
                'message' => 'Register Successfully'
            ]);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Post(
     *     path="/api/logout",
     *     operationId="Logout",
     *     summary="Logout",
     *     description="Logout",
     *     security={
     *           {"bearerAuth": {}}
     *     },
    * @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *     @OA\Schema(
     *         type="object",
     *         @OA\Property(property="device_type", type="staring"),
     *         @OA\Property(property="device_token", type="staring")
     *       ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Logged out Successfully",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function logout(Request $request)
    {
        $data = $request->all();

        if (isset( $data['device_type']) && isset($data['device_token']) && \App\Models\DeviceToken::where([
            'app_user_id' => Auth::guard('api')->user()->app_user_id,
            'device_type' => $data['device_type'],
            'device_token' => $data['device_token']
        ])->exists() == true) {
            \App\Models\DeviceToken::where([
                'app_user_id' => Auth::guard('api')->user()->app_user_id,
                'device_type' => $data['device_type'],
                'device_token' => $data['device_token']
            ])->delete();
        }

        auth()->guard('api')->logout();


        return response()->json([
            'status' => true,
            'message' => 'Logged out Successfully'
        ]);
    }

    /**
     * @OA\Post(
     *     path="/api/edit-profile",
     *     operationId="editProfile",
     *     summary="Edit Profile",
     *     description="Edit Profile",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={"first_name","last_name","email","company","title","phone_no","mailing_address","country_code"},
     *                 @OA\Property(property="first_name", type="staring"),
     *                 @OA\Property(property="last_name", type="staring"),
     *                 @OA\Property(property="email", type="email"),
     *                 @OA\Property(property="company", type="staring"),
     *                 @OA\Property(property="title", type="staring"),
     *                 @OA\Property(property="country_code", type="staring"),
     *                 @OA\Property(property="phone_no", type="number"),
     *                 @OA\Property(property="mailing_address", type="staring"),
     *                 @OA\Property(property="assistant_name", type="staring"),
     *                 @OA\Property(property="assistant_email", type="email"),
     *                 @OA\Property(property="assistant_country_code", type="staring"),
     *                @OA\Property(property="assistant_phone_no", type="number"),
     *                 @OA\Property(property="birthday", type="date"),
     *                 @OA\Property(property="personal_interests", type="strings"),
     *                 @OA\Property(property="business_interests", type="strings"),
     *                 @OA\Property(property="spouse_name", type="strings"),
     *                 @OA\Property(property="spouse_birthday", type="date"),
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Edit Profile.",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function editProfile(Request $request)
    {
        try {
            $id =  Auth::guard('api')->user()->app_user_id;

            // Validate user input
            $validator = Validator::make($request->all(), [
                'first_name' => 'required',
                // 'last_name' => 'required',
                'last_name' => 'nullable',
                'email' => ['required', 'email' , Rule::unique('app_users')->whereNull('deleted_at')->whereNot('app_user_id', $id)],
                // 'company' => 'required',
                // 'title' => 'required',
                // 'phone_no' => ['required', 'numeric', 'digits:10' , Rule::unique('app_users')->whereNull('deleted_at')->whereNot('app_user_id', $id)],
                // 'mailing_address' => 'required',
                'company' => 'nullable',
                 'title' => 'nullable',
                // 'phone_no' => ['nullable', 'numeric', 'digits:10' , Rule::unique('app_users')->whereNull('deleted_at')->whereNot('app_user_id', $id)],
                'mailing_address' => 'nullable',
                
                'assistant_name' => 'nullable',
                'assistant_email' => 'nullable|email',
                'assistant_phone_no' => 'nullable|numeric|digits:10',
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $existsCompany = Company::whereRaw('LOWER(`company_name`) = ? ', [trim(strtolower($request->company))])->first();
            $companyId = null;
            if (!empty($request->company)) {
                if (is_null($existsCompany)) {
                    
                    $company = Company::create([
                        'company_name' => ($request->company),
                        'company_status' => 1
                    ]);
                    $companyId = $company->company_id;
                } else {
                    $companyId = $existsCompany->company_id;
                }
            }
            $updateAppUser = [
                'first_name' => $request->first_name,
                'last_name' => ($request->last_name) ? $request->last_name : '',
                'email' => $request->email,
                'company' => ($companyId) ? $companyId : null,
                'title' => ($request->title) ? $request->title : null ,
                'country_code' => isset($request->country_code) ? $request->country_code : 'US',
                'phone_no' => ($request->phone_no) ? $request->phone_no : 0000000000,
                'mailing_address' => ($request->mailing_address) ? $request->mailing_address : null,
                'assistant_name' => isset($request->assistant_name) ? $request->assistant_name : '',
                'assistant_email' => isset($request->assistant_email) ? $request->assistant_email : '',
                'assistant_country_code' => isset($request->assistant_country_code) ? $request->assistant_country_code : 'US',
                'assistant_phone_no' => isset($request->assistant_phone_no) ? $request->assistant_phone_no : '',
                'birthday' => isset($request->birthday) ? $request->birthday : NULL,
                'personal_interests' => isset($request->personal_interests) ? $request->personal_interests : NULL,
                'business_interests' => isset($request->business_interests) ? $request->business_interests : NULL,
                'spouse_name' => isset($request->spouse_name) ? $request->spouse_name : NULL,
                'spouse_birthday' => isset($request->spouse_birthday) ? $request->spouse_birthday : NULL,
            ];

            if (isset($request->profile_img) || !empty($request->profile_img)) {
                $binary_data = base64_decode($request->profile_img);
                $image_name = time() . '.png';

                $directory = public_path('/profile_image/');

                if (!is_dir($directory)) {
                    mkdir($directory, 0777, true);
                }
                file_put_contents($directory . $image_name, $binary_data);
                $updateAppUser['profile_img']  = $image_name;

                if(Auth::guard('api')->user()->profile_img != ''){
                    unlink(public_path('/profile_image/'). Auth::guard('api')->user()->profile_img);
                }
            }

            // Create the user
            $user = AppUser::where('app_user_id', $id)->update($updateAppUser);

            \App\Models\AppUserChildDetail::where('app_user_id', $id)->delete();
            if (isset($request->children)) {
                $children = [];
                foreach ($request->children as $key => $val) {
                    $val['app_user_id'] = $id;
                    $val['birthday'] = date('Y-m-d', strtotime($val['birthday']));
                    $children[] = $val;
                }
                \App\Models\AppUserChildDetail::insert($children);
            }


            return response()->json([
                'status' => true,
                'message' => 'Edit Profile.'
            ]);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Get(
     *     path="/api/profile",
     *     operationId="profileDetail",
     *     summary="Profile",
     *     description="Profile",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\Response(
     *         response=200,
     *         description="Profile Detail",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function profileDetail(Request $request)
    {
        try {

            $user = Auth::guard('api')->user();

            $companyDetail = \App\Models\Company::select('company_id', 'company_name')->where('company_id', Auth::guard('api')->user()->company)->first();
            $user['company'] = $companyDetail;

            $user['subscription'] = [];
            if (!is_null($user->subscription_id) || $user->subscription_id != '') {
                $subscription = Subscription::select('subscription_level', 'subscription_name', 'subscription_content', 'subscription_price_per_month', 'subscription_price_per_year', 'subscription_status')->where('subscription_id', $user->subscription_id)->first();
                $user['subscription'] = !is_null($subscription) ? $subscription->toArray() : [];
            }

            $user['subscription_id'] = !empty($user['subscription_id']) ? $user['subscription_id']  :  null;
            $user['children'] = $user->children;

            if (!empty($user)) {
                $response = [
                    'data' => $user,
                    'status' => true,
                    'message' => __('messages.detail', ['name' => 'User profile'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }

            return response()->json($response);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Post(
     *     path="/api/social/login",
     *     operationId="socialLogin",
     *     summary="Social Login",
     *     description="Social Login",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={ "social_id", "social_type"},
     *                 @OA\Property(property="social_id", type="string"),
     *                 @OA\Property(property="social_type", type="string"),
     *                 @OA\Property(property="first_name", type="string"),
     *                 @OA\Property(property="last_name", type="string"),
     *                 @OA\Property(property="email", type="string"),
     *                 @OA\Property(property="phone_no", type="string")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Login",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function socialLogin(Request $request)
    {
        try {

            $validator = Validator::make($request->all(), [
                'social_id' => 'required',
                'social_type' => 'required',
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $data = $request->all();
            $email = $data['email'] ?? '';
            $phoneNo = $data['phone_no'] ?? '';
            $countryCode = $data['country_code'] ?? 'US';
            $firstName = $data['first_name'] ?? '';
            $lastName = $data['last_name'] ?? '';
            $socialType = $data['social_type'] ?? '';
            $socialId = $data['social_id'] ?? '';

            $detail = [
                'email' => $email,
                'country_code' => $countryCode,
                'phone_no' => $phoneNo,
                'first_name' => $firstName,
                'last_name' => $lastName,
            ];

            if ($socialType == 'facebook') {
                $detail['facebook_id'] = $socialId;
            } else if ($socialType == 'linkedin') {
                $detail['linkedin_id'] = $socialId;
            } else {
                $detail['google_id'] = $socialId;
            }

            $user = AppUser::where(function ($query) use ($socialId) {
                $query->where('facebook_id', $socialId);
                $query->orWhere('google_id', $socialId);
                $query->orWhere('linkedin_id', $socialId);
            })->first();

            if (is_null($user)) {
                // if ($email != '') {
                //     $user = AppUser::where(['email' => $detail['email']])->first();
                //     if (!is_null($user)) {
                //     AppUser::where(['email' => $detail['email']])->update($detail);
                //     } else {
                //         AppUser::insert($detail);
                //     }
                // } else if ($phoneNo != '') {
                //     $user = AppUser::where(['phone_no' => $detail['phone_no']])->first();
                //     if (!is_null($user)) {
                //     AppUser::where(['phone_no' => $detail['phone_no']])->update($detail);
                //     } else {
                //         AppUser::insert($detail);
                //     }
                // } else {
                //     AppUser::insert($detail);
                // }
                return response()->json([
                    'data' => (object) [],
                    'status' => false,
                    'message' => 'User Is Not Found',
                ]);
            }else{
                $user = AppUser::where(function ($query) use ($socialId) {
                    $query->where('facebook_id', $socialId);
                    $query->orWhere('google_id', $socialId);
                    $query->orWhere('linkedin_id', $socialId);
                })->first();
    
                $token = Auth::guard('api')->login($user);
    
                if (!$token) {
                    return response()->json([
                        'status' => 'error',
                        'message' => 'Unauthorized',
                    ], 401);
                }
    
                $user = Auth::guard('api')->user();
    
                $user['company'] = (object) [];
                $user['subscription'] = (object) [];
    
                return response()->json([
                    'status' => true,
                    'message' => 'Login Successfully',
                    'user' => $user,
                    'token' => $token
                ]);
            }

        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Post(
     *     path="/api/forgot/password",
     *     operationId="forgotPassword",
     *     summary="Forgot Password",
     *     description="Forgot Password",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={"email"},
     *                 @OA\Property(property="email", type="staring", example="joshua.manning@gmail.com"),
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Forgot Password.",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function forgotPassword(Request $request)
    {

        try {

            $validator = Validator::make($request->all(), [
                'email' => 'required|email|exists:app_users',
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $token = Str::random(64);

            DB::table('app_user_password_resets')->insert([
                'email' => $request->email,
                'token' => $token,
                'created_at' => Carbon::now()
            ]);

            $link = 'https://www.shopsecured.com/luxury_app/public/app-reset-password/'.$token .'?email='. $request->email;

            //$link = route('app.reset.password.get', [$token, 'email' => $request->email]);

            Mail::send('mails.forgetPassword', ['link' => $link], function ($message) use ($request) {
                $message->to($request->email);
                $message->subject('Luxury Council - Reset Password');
            });


            return response()->json([
                'status' => true,
                'message' => 'Please check register email id.',
                'data' => (object) [],
            ]);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => 'Error in sending email; contact administrator for help.'
            ];
            return response()->json($response);
        }
    }


    /**
     * @OA\Post(
     *     path="/api/manage-device-token",
     *     operationId="manageDeviceToken",
     *     summary="Manage Device Token",
     *     description="Manage Device Token",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 required={"device_type", "device_token", "is_delete"},
     *                 @OA\Property(property="device_type", type="staring"),
     *                 @OA\Property(property="device_token", type="staring"),
     *                 @OA\Property(property="is_delete", type="staring")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Manage Device Token.",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function manageDeviceToken(Request $request)
    {

        try {

            $data = $request->all();

            $validator = Validator::make($data, [
                'device_token' => 'required',
                'device_type' => 'required',
                'is_delete' => 'required'
            ]);

            if ($validator->fails()) {
                $validation = [
                    'data' => (object) [],
                    'status' => false,
                    'message' =>  $validator->errors()->first()
                ];
                return response()->json($validation);
            }

            $exists = \App\Models\DeviceToken::where([
                'app_user_id' => Auth::guard('api')->user()->app_user_id,
                'device_type' => $data['device_type'],
                'device_token' => $data['device_token']
            ])->exists();
            if ($exists ==  true && $data['is_delete'] == 1) {
                \App\Models\DeviceToken::where([
                    'app_user_id' => Auth::guard('api')->user()->app_user_id,
                    'device_type' => $data['device_type'],
                    'device_token' => $data['device_token']
                ])->delete();
            } else if ($exists ==  false && $data['is_delete'] == 0) {
                \App\Models\DeviceToken::insert([
                    'app_user_id' => Auth::guard('api')->user()->app_user_id,
                    'device_type' => ($request->input('device_type') == 0 ) ? 1 : $request->input('device_type'),
                    'device_token' => $request->input('device_token')
                ]);
            }

            return response()->json([
                'status' => true,
                'message' => 'Device token.',
                'data' => (object) [],
            ]);
        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }
}
