<?php

namespace App\Http\Controllers\API;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Hash;
use App\Models\Company;
use App\Models\Category;
use App\Models\Industry;
use App\Models\Interest;
use App\Models\Publisher;
use App\Models\Subscription;
class MasterController extends Controller
{
    public function __construct() {
        $this->middleware('auth', ['except' => [
            'companyList', 'interestList'
        ]]);
    }

    /**
     * @OA\Post(
     *     path="/api/company-list",
     *     operationId="companyList",
     *     summary="Company List",
     *     description="Company List",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 @OA\Property(property="page", type="integer"),
     *                 @OA\Property(property="limit", type="integer")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Company List",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function companyList(Request $request)
    {
        try {
            $page = $request->input('page') ?? 1;
            $limit = $request->input('limit') ?? 10;
            $offset = $this->offset($page, $limit);

            $company = Company::select('company_id', 'company_name', 'company_status')->orderBy('company_name', 'ASC')->skip($offset)->take($limit)->get()->toArray();

            if (!empty($company)) {
                $response = [
                    'data' => $company,
                    'status' => true,
                    'message' => __('messages.list', ['name' => 'Company'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }

            return response()->json($response);

        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Post(
     *     path="/api/category-list",
     *     operationId="categoryList",
     *     summary="Category List",
     *     description="Category List",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 @OA\Property(property="page", type="integer"),
     *                 @OA\Property(property="limit", type="integer")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Category List",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function categoryList(Request $request)
    {
        try {
            $page = $request->input('page') ?? 1;
            $limit = $request->input('limit') ?? 10;
            $offset = $this->offset($page, $limit);

            $category = Category::select('category_id', 'category_name', 'category_status')->status(1)->orderBy('category_name', 'ASC')->skip($offset)->take($limit)->get()->toArray();

            if (!empty($category)) {
                $response = [
                    'data' => $category,
                    'status' => true,
                    'message' => __('messages.list', ['name' => 'Category'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }

            return response()->json($response);

        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Post(
     *     path="/api/industry-list",
     *     operationId="industryList",
     *     summary="Industry List",
     *     description="Industry List",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 @OA\Property(property="page", type="integer"),
     *                 @OA\Property(property="limit", type="integer")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Industry List",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function industryList(Request $request)
    {
        try {
            $page = $request->input('page') ?? 1;
            $limit = $request->input('limit') ?? 10;
            $offset = $this->offset($page, $limit);

            $industry = Industry::select('industry_id', 'industry_name', 'industry_status')->orderBy('industry_name', 'ASC')->skip($offset)->take($limit)->get()->toArray();

            if (!empty($industry)) {
                $response = [
                    'data' => $industry,
                    'status' => true,
                    'message' => __('messages.list', ['name' => 'Industry'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }

            return response()->json($response);

        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Post(
     *     path="/api/interest-list",
     *     operationId="interestList",
     *     summary="Interest List",
     *     description="Interest List",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 @OA\Property(property="page", type="integer"),
     *                 @OA\Property(property="limit", type="integer")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Interest List",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function interestList(Request $request)
    {
        try {
            $page = $request->input('page') ?? 1;
            $limit = $request->input('limit') ?? 10;
            $offset = $this->offset($page, $limit);

            $industry = Interest::select('interest_id', 'interest_name', 'interest_status')->status(1)->orderBy('interest_name', 'ASC')->skip($offset)->take($limit)->get()->toArray();

            if (!empty($industry)) {
                $response = [
                    'data' => $industry,
                    'status' => true,
                    'message' => __('messages.list', ['name' => 'Interest'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }

            return response()->json($response);

        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Post(
     *     path="/api/publisher-list",
     *     operationId="publisherList",
     *     summary="Publisher List",
     *     description="Publisher List",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\RequestBody(
     *         required=true,
     *         @OA\MediaType(
     *             mediaType="multipart/form-data",
     *             @OA\Schema(
     *                 type="object",
     *                 @OA\Property(property="page", type="integer"),
     *                 @OA\Property(property="limit", type="integer")
     *             ),
     *         ),
     *     ),
     *     @OA\Response(
     *         response=200,
     *         description="Publisher List",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function publisherList(Request $request)
    {
        try {
            $page = $request->input('page') ?? 1;
            $limit = $request->input('limit') ?? 10;
            $offset = $this->offset($page, $limit);

            $industry = Publisher::select('publisher_id', 'publisher_name', 'publisher_status')->orderBy('publisher_name', 'ASC')->skip($offset)->take($limit)->get()->toArray();

            if (!empty($industry)) {
                $response = [
                    'data' => $industry,
                    'status' => true,
                    'message' => __('messages.list', ['name' => 'Publisher'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }

            return response()->json($response);

        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Get(
     *     path="/api/media-list",
     *     operationId="mediaList",
     *     summary="Media List",
     *     description="Media List",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\Response(
     *         response=200,
     *         description="Media List",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function mediaList(Request $request)
    {
        try {
            $media = [
                [
                    'media_id' => 'Article',
                    'media_name' => 'Article',
                    'media_status' => 1
                ], [
                    'media_id' => 'Resource',
                    'media_name' => 'Resource',
                    'media_status' => 1
                ], [
                    'media_id' => 'Audio',
                    'media_name' => 'Audio',
                    'media_status' => 1
                ], [
                    'media_id' => 'Video',
                    'media_name' => 'Video',
                    'media_status' => 1
                ], [
                    'media_id' => 'Image',
                    'media_name' => 'Image',
                    'media_status' => 1
                ]
            ];

            if (!empty($media)) {
                $response = [
                    'data' => $media,
                    'status' => true,
                    'message' => __('messages.list', ['name' => 'Media'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }

            return response()->json($response);

        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }

    /**
     * @OA\Get(
     *     path="/api/subscription-list",
     *     operationId="subscriptionList",
     *     summary="Subscription List",
     *     description="Subscription List",
     *     security={
     *           {"bearerAuth": {}}
     *     },
     *     @OA\Response(
     *         response=200,
     *         description="Subscription List",
     *         @OA\JsonContent()
     *     )
     * )
     */
    public function subscriptionList(Request $request)
    {
        try {
            $subscription = Subscription::select('subscription_id', 'subscription_level', 'subscription_name', 'subscription_content', 'subscription_price_per_month', 'subscription_price_per_year', 'subscription_status')->status(1)->orderBy('subscription_id', 'ASC')->get()->toArray();

            foreach($subscription as $key => $val){
                $subscription[$key]['subscription_price_per_month'] = number_format($subscription[$key]['subscription_price_per_month'], 2);
                $subscription[$key]['subscription_price_per_year'] = number_format($subscription[$key]['subscription_price_per_year'], 2);
            }

            if (!empty($subscription)) {
                $response = [
                    'data' => $subscription,
                    'status' => true,
                    'message' => __('messages.list', ['name' => 'Subscription'])
                ];
            } else {
                $response = [
                    'data' => (object) [],
                    'status' => true,
                    'message' => __('messages.not_found')
                ];
            }

            return response()->json($response);

        } catch (\Exception $e) {

            $response = [
                'data' => (object) [],
                'status' => false,
                'message' => $e->getMessage()
            ];
            return response()->json($response);
        }
    }
}
