<?php

namespace App\Http\Controllers;

use DataTables;
use Carbon\Carbon;
use App\Models\Category;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\DB;
use App\Rules\UniqueCategoryName;
use App\Rules\CategoryStatusChange;

class CategoryController extends Controller
{
    public function list(Request $request)
    {
        return view('category.indexCategory');
    }
    public function index(Request $request,$id=0)
    {
       if(!empty($id)){
            $category=Category::where('category_id',$id)->first();
        }else{
            $category='';
        }
        return view('category.createCategory', compact('category'));
    }
    public function addCategory(Request $request,$id=0)
    {

        $data=$request->all();
        if($data['category_id']!=null)
            {
                $this->validate($request, [
                'category_name'=> ['required', new UniqueCategoryName($data['category_id'])],
                // 'category_status' => new CategoryStatusChange($data['category_id'])
                ]);

            unset($data['_token']);
            Category::where('category_id', $data['category_id'])->update($data);
            return redirect()->route('category.list')->with('message', __('messages.edit', ['name' => 'Category']));
        }else{
            $this->validate($request, [
                'category_name'=> ['required', new UniqueCategoryName(0)],
            ]);

            unset($data['category_id']);
            Category::create($data);
            return redirect()->route('category.list')->with('message',__('messages.add', ['name' => 'Category']));
        }

    }
    public function deleteCategory($id)
    {
        $artical = \App\Models\Article::whereJsonContains('category_id',$id)->count();
        $check = DB::table('articles')->whereJsonContains('category_id',$id)->whereNull('deleted_at')->first();
        if($artical > 0)
        {
            return redirect()->route('category.list')->with('message_wr',__('messages.already_use', ['name' => 'Category']));
        }else{
            Category::where('category_id', $id)->update(['deleted_at' => Carbon::now()]);
            return redirect()->route('category.list')->with('message', __('messages.delete', ['name' => 'Category']));
        }
        
    }

    public function ajaxList(Request $request)
    {
        if($request->ajax()){
            $columns = ['action','category_name', 'category_status'];
            $columnIndex = $request->input('order.0.column');
            $columnName = $columns[$columnIndex];
            $columnSortOrder = $request->input('order.0.dir');

            $data_category = Category::orderBy($columnName, $columnSortOrder);

            $data_category->get();
            return DataTables::of($data_category)
            ->addColumn('category_status', function ($row) {
                if($row->category_status==1)
                {
                    $category_status="Active";
                }else{
                    $category_status="Inactive";
                }
                return $category_status;
            })
            ->addColumn('action', function ($row) {
                $editUrl = route('category.form', $row->category_id);
                $deleteUrl = route('category.delete', $row->category_id);
                return '<a href="'.$editUrl.'" class="btn" style="background: #000000;color: #f0ac63 !important;" title="Edit"><i class="fa fa-pencil" aria-hidden="true"></i></a>
                        <a href="'.$deleteUrl.'" class="btn" onclick="return confirm(\'Are you sure you want to delete this Category?\')" style="background: #f0ac63;color: #000000 !important;" title="Delete"><i class="fa fa-trash" aria-hidden="true"></i></a>';
            })
            ->rawColumns(['category_status','action'])
            ->addIndexColumn()
            ->make(true);
        }
    }

}
