<?php

namespace App\Http\Controllers;

use DataTables;
use Carbon\Carbon;
use App\Models\Company;

use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\DB;
use App\Rules\UniqueCompanyName;
use App\Rules\CompanyStatusChange;

class CompanyController extends Controller
{
    public function index(Request $request,$id=0)
    {
       
        if(!empty($id)){
            $company=Company::where('company_id',$id)->first();
        }else{
            $company='';
        }
        return view('company.createCompany', compact('company'));
    }
    public function addCompany(Request $request,$id=0)
    {
        $data=$request->all();
        

        

        if($data['company_id']!=null)
        {

                $this->validate($request, [
                'company_name'=> ['required', new UniqueCompanyName($data['company_id'])],
                // 'company_status' => new CompanyStatusChange($data['company_id'])
                ]);

            unset($data['_token']);
            Company::where('company_id', $data['company_id'])->update($data);
            return redirect()->route('company.list')->with('message',__('messages.edit', ['name' => 'Company']));
        }else{
            $this->validate($request, [
                'company_name'=> ['required', new UniqueCompanyName(0)]
            ]);

            unset($data['company_id']);
            Company::create($data);
            return redirect()->route('company.list')->with('message', __('messages.add', ['name' => 'Company']));
        }

    }
    public function list(Request $request)
    {

        return view('company.indexCompany');
    }

    public function deleteCompany($id)
    {

        $appUser  = \App\Models\AppUser::where('company', $id)->count();
        $groupSubscription  = \App\Models\GroupSubscription::where('company_name', $id)->count();

        if($appUser > 0 || $groupSubscription > 0)
        {
            return redirect()->route('company.list')->with('message_wr', __('messages.already_use', ['name' => 'Company']));
        }else{
            company::where('company_id', $id)->update(['deleted_at' => Carbon::now()]);
            return redirect()->route('company.list')->with('message', __('messages.delete', ['name' => 'Company']));
        }

        
        
    }

    public function ajaxList(Request $request)
    {
        if($request->ajax()){

            $columns = ['action','company_name', 'company_status'];
            $columnIndex = $request->input('order.0.column');
            $columnName = $columns[$columnIndex];
            $columnSortOrder = $request->input('order.0.dir');

            $data_company = Company::orderBy($columnName, $columnSortOrder);
            $data_company->get();
            return DataTables::of($data_company)
            ->addColumn('company_status', function ($row) {
                if($row->company_status==1)
                {
                    $company_status="Active";
                }else{
                    $company_status="Inactive";
                }
                return $company_status;
            })
            ->addColumn('action', function ($row) {
                $editUrl = route('company.form', $row->company_id);
                $deleteUrl = route('company.delete', $row->company_id);
                return '<a href="'.$editUrl.'" class="btn" style="background: #000000;color: #f0ac63 !important;"><i class="fa fa-pencil" aria-hidden="true" title="Edit"></i></a>
                        <a href="'.$deleteUrl.'" class="btn" onclick="return confirm(\'Are you sure you want to delete this Company?\')" style="background: #f0ac63;color: #000000 !important;" title="Delete"><i class="fa fa-trash" aria-hidden="true"></i></a>';
            })
            ->rawColumns(['company_status','action'])
            ->addIndexColumn()
            ->make(true);
        }
    }
}
