<?php

namespace App\Http\Controllers;

use Illuminate\Foundation\Auth\Access\AuthorizesRequests;
use Illuminate\Foundation\Bus\DispatchesJobs;
use Illuminate\Foundation\Validation\ValidatesRequests;
use Illuminate\Routing\Controller as BaseController;
use Illuminate\Pagination\Paginator;
use Illuminate\Support\Collection;
use Illuminate\Pagination\LengthAwarePaginator;

 /**
     * @OA\Info(
     *    title="Luxury App",
     *    version="1.0.0",
     * ),
     *
     *@OA\SecurityScheme(
     *  securityScheme="bearerAuth",
     *  in="header",
     *  name="bearerAuth",
     *  type="http",
     *  scheme="bearer",
     *  bearerFormat="JWT",
     *),
    */
class Controller extends BaseController
{
    use AuthorizesRequests, DispatchesJobs, ValidatesRequests;

    public function offset($page, $limit)
    {
        return ($page - 1) * $limit;
    }

    public function getArticleAfterApplyVisiableDate()
    {
        $query =  \App\Models\Article::select('article_id')->whereRaw("JSON_CONTAINS(subscription_level, '\"2\"')");
       
        $query->whereRaw("DATE(now()) < DATE_ADD(publish_date, INTERVAL visiable_after_days DAY)");
        return $query->whereNotNull('visiable_after_days')->get()->pluck('article_id');
    }

    public function paginate($items, $perPage = 5, $page = null, $options = [])
    {
        $currentPage = LengthAwarePaginator::resolveCurrentPage();
        $collection = new Collection($items);
        $currentPageResults = $collection->slice(($page - 1) * $perPage, $perPage)->values();
        $data['results'] = new LengthAwarePaginator($currentPageResults, count($collection), $perPage);
        return $data['results']->setPath(request()->url());
    }
}
