<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Event;
use DataTables;
use Carbon\Carbon;
use App\Models\User;
use App\Jobs\SendEventMailJob;
use App\Models\AppUser;

class EventController extends Controller
{
    public function __construct()
    {
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        return view('event.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('event.form');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $data = $request->all();

        $rules = [
            'event_name' => 'required',
            'event_date' => 'required',
            'event_detail' => 'required',
            'hash_tags' => 'required',
            'event_email_subject' => 'required',
            'event_email_body' => 'required'
        ];

        $request->validate($rules);

        $eventData = [
            "event_name" => $data['event_name'],
            "event_date" => $data['event_date'],
            "event_detail" =>  $data['event_detail'],
            "hash_tags" =>  $data['hash_tags'],
            "event_email_subject" =>  $data['event_email_subject'],
            "event_email_body" =>  $data['event_email_body'],
            "event_status" =>  $data['event_status']
        ];

        Event::create($eventData);
        return redirect()->route('event.index')->with('message', __('messages.add', ['name' => 'Event']));

    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $event = Event::where('event_id', $id)->first();
        return view('event.form', ['event' => $event]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $data = $request->all();

        $rules = [
            'event_name' => 'required',
            'event_date' => 'required',
            'event_detail' => 'required',
            'hash_tags' => 'required',
            'event_email_subject' => 'required',
            'event_email_body' => 'required'
        ];

        $request->validate($rules);

        $eventData = [
            "event_name" => $data['event_name'],
            "event_date" => $data['event_date'],
            "event_detail" =>  $data['event_detail'],
            "hash_tags" =>  $data['hash_tags'],
            "event_email_subject" =>  $data['event_email_subject'],
            "event_email_body" =>  $data['event_email_body'],
            "event_status" =>  $data['event_status']
        ];

        Event::where('event_id', $id)->update($eventData);
        return redirect()->route('event.index')->with('message',  __('messages.edit', ['name' => 'Event']));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        Event::where('event_id', $id)->update(['deleted_at' => Carbon::now()]);
        return redirect()->route('event.index')->with('message', __('messages.delete', ['name' => 'Event']));
    }

    public function ajaxList(Request $request)
    {

        if ($request->ajax()) {

            $columns = ['action', 'event_date', 'event_name', 'event_detail', 'hash_tags', 'event_status'];
            $columnIndex = $request->input('order.0.column');
            $columnName = $columns[$columnIndex];
            $columnSortOrder = $request->input('order.0.dir');

            $data_spotlight_member = Event::select('event_id', 'event_date', 'event_detail', 'hash_tags', 'event_email_subject', 'event_status', 'event_name')
            ->orderBy($columnName, $columnSortOrder);
            $data_spotlight_member->get();

            return DataTables::of($data_spotlight_member)
                ->addColumn('event_status', function ($row) {
                    return $row->event_status == 1 ? "Active" : "Inactive";
                })
                ->addColumn('action', function ($row) {
                    $editUrl = route('event.edit', $row->event_id);
                    $deleteUrl = route('event.destroy', $row->event_id);
                    $preview = '<button type="button" class="btn btn-light" data-toggle="modal" data-target="#exampleModal" data-event_id="'.addslashes($row->event_id).'"  title="Preview"><i class="fa fa-eye" aria-hidden="true"></i></button>';
                    $sendMail = '<button title="Send Mail" type="button" class="btn btn-light" data-toggle="modal" data-target="#sendMail" data-event_id="'.addslashes($row->event_id).'" style="background: #218838;" title="Send Mail"><i class="fa fa-envelope" aria-hidden="true"></i>
                    </button>';

                    return '<a href="' . $editUrl . '" class="btn" style="background: #000000;color: #f0ac63 !important;" title="Edit"><i class="fa fa-pencil" aria-hidden="true"></i></a>

                    <form action="'.route('event.destroy', $row->event_id).'" method="post" style="display:inline;" id="delete_'. $row->event_id .'">
                        <button class="btn" type="button" style="background: #f0ac63;color: #000000 !important;" onclick="return deleteRecord($(this))" data-url="'.$row->event_id.'" title="Delete"/><i class="fa fa-trash" aria-hidden="true"></i></button>
                        <input type="hidden" name="_method" value="delete" />
                        <input type="hidden" name="_token" value="'. csrf_token().'">
                    </form> ' . $preview . ' ' . $sendMail;
                })
                ->filter(function ($query) use ($request) {

                    if (isset($request->event_name)) {
                        $search = '%' . $request->event_name . '%';
                        $query->where("event_name", "LIKE", $search);
                    }
                    if (isset($request->event_date)) {
                        $event_date = date('Y-m-d', strtotime($request->event_date));
                        $query->where("event_date", $event_date);
                    }
                    if (isset($request->hash_tags)) {
                        $search = '%' . $request->hash_tags . '%';
                        $query->where("hash_tags", "LIKE", $search);
                    }
                    if (isset($request->event_email_subject)) {
                        $search = '%' . $request->event_email_subject . '%';
                        $query->where("event_email_subject", "LIKE", $search);
                    }
                    if (isset($request->event_detail)) {
                        $search = '%' . $request->event_detail . '%';
                        $query->where("event_detail", "LIKE", $search);
                    }
                })
                ->rawColumns(['action', 'event_status', 'event_detail'])
                ->addIndexColumn()
                ->make(true);
        }
    }

    public function preview(Request $request)
    {
        $event_id = $request->event_id;
        $event_data = Event::select('event_name', 'event_date', 'event_email_subject', 'event_email_body')->where('event_id', $event_id)->first();
        return response()->json(['event_data' => view('event.preview',['event_data'=> $event_data])->render()]);
    }

    public function openOverlay(Request $request)
    {
        $event_id = $request->input('event_id');
        return response()->json(['html' => view('event.send-mail-overlay',[ 'event_id' => $event_id])->render()]);
    }

    public function sendMail(Request $request)
    {
        $event_id = $request->input('event_id');
        $emails = $request->input('emails');
        try {
        $event_detail = Event::select('event_email_subject', 'event_email_body')->where('event_id',  $event_id)->first();
        $details = [
            'emails' => $emails,
            'event_email_subject' => $event_detail->event_email_subject,
            'event_email_body' => $event_detail->event_email_body
        ];

        dispatch(new SendEventMailJob($details));
        \App\Models\EventMailLog::create(['event_id' => $event_id, 'emails' => json_encode($emails)]);
            return response()->json(['success' => 'Mail Send Successfully!!']);
        } catch (\Exception $e) {
            return response()->json(['error' => $e->getMessage()]);
        }
    }

    public function searchAppUser(Request $request)
    {
        $company_name = $request->input('company_name');
        $title = $request->input('title');
        $event_id = $request->input('event_id');

        $query = AppUser::select(['app_users.*', 'company.*'])->leftJoin('company', 'company.company_id', '=', 'app_users.company')->where('app_users.email', '<>', "");
        if($company_name != ''){
            $query->where("company.company_name", "LIKE", '%'.$company_name.'%');
        }
        if($title != ''){
            $query->where("title", "LIKE",  '%'.$title.'%');
        }
        $appUserDetail = $query->get();
        return response()->json(['html' => view('event.search-app-user',['appUserDetail' => $appUserDetail, 'event_id' => $event_id])->render()]);

    }
}
