<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\GroupSubscription;
use App\Models\Subscription;
use DataTables;
use Carbon\Carbon;
use Validator;
use App\Imports\UsersImport;
use Maatwebsite\Excel\Facades\Excel;
use App\Models\ImportUser;
use App\Jobs\ImportAppUser;
use App\Models\AppUser;
use Illuminate\Support\Facades\DB;
use App\Models\Company;
use Illuminate\Validation\Rule;
use App\Models\GroupSubscriptionTransactionHistory;
use App\Jobs\AssignGroupSubscription;

class GroupSubscriptionController extends Controller
{
    public function __construct()
    {
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        $subscripation = Subscription::pluck('subscription_name', 'subscription_id');
        return view('group-subscription.index', ['subscripation' => $subscripation]);
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $country = \App\Models\Country::select(DB::raw('CONCAT(short_name, " ", "(+", code, ")") AS short_code'), 'short_name')->pluck('short_code', 'short_name');
        $company = Company::orderBy("company_name", "asc")->pluck('company_name', 'company_id');
        return view('group-subscription.form', ['company' => $company, 'country' => $country]);
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $data = $request->all();

        $rules = [
            'company_name' => 'required',
            'contact_name' => 'required',
            'title' => 'required',
            'email' => ['required', 'email', Rule::unique('group_subscriptions')->whereNull('deleted_at')],
            'country_code' => 'required',
            'phone_no' => ['required', 'numeric', 'digits:10', Rule::unique('group_subscriptions')->whereNull('deleted_at')],
            'number_of_subscription' => 'required',
            'group_subscription_status' => 'required',
        ];

        $request->validate($rules);

        $groupSubscriptionData = [
            "company_name" => $data['company_name'],
            "contact_name" =>  $data['contact_name'],
            "email" =>  $data['email'],
            "title" =>  $data['title'],
            "country_code" =>  $data['country_code'],
            "phone_no" =>  $data['phone_no'],
            "number_of_subscription" =>  $data['number_of_subscription'],
            "group_subscription_status" =>  $data['group_subscription_status'],
        ];

        GroupSubscription::create($groupSubscriptionData);

        return redirect()->route('group.subscription.index')->with('message', __('messages.add', ['name' => 'Group Subscription']));
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $country = \App\Models\Country::select(DB::raw('CONCAT(short_name, " ", "(+", code, ")") AS short_code'), 'short_name')->pluck('short_code', 'short_name');
        $company =  Company::orderBy("company_name", "asc")->pluck('company_name', 'company_id');
        $groupSubscription = GroupSubscription::where('group_subscription_id', $id)->first();
        return view('group-subscription.form', ['groupSubscription' => $groupSubscription, 'company' => $company, 'country' => $country]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $data = $request->all();

        $rules = [
            'company_name' => 'required',
            'contact_name' => 'required',
            'title' => 'required',
            'email' => ['required', 'email', Rule::unique('group_subscriptions')->whereNull('deleted_at')->whereNot('group_subscription_id', $data['group_subscription_id'])],
            'country_code' => 'required',
            'phone_no' => ['required', 'numeric', 'digits:10', Rule::unique('group_subscriptions')->whereNull('deleted_at')->whereNot('group_subscription_id', $data['group_subscription_id'])],
            'number_of_subscription' => 'required',
            'group_subscription_status' => 'required',
        ];

        $request->validate($rules);

        $groupSubscriptionData = [
            "company_name" => $data['company_name'],
            "contact_name" =>  $data['contact_name'],
            "email" =>  $data['email'],
            "title" =>  $data['title'],
            "country_code" =>  $data['country_code'],
            "phone_no" =>  $data['phone_no'],
            "number_of_subscription" =>  $data['number_of_subscription'],
            "group_subscription_status" =>  $data['group_subscription_status'],
        ];

        GroupSubscription::where('group_subscription_id', $id)->update($groupSubscriptionData);

        \App\Models\AppUser::where('group_subscription_id', $id)->update(["app_user_status" => $data['group_subscription_status']]);

        return redirect()->route('group.subscription.index')->with('message', __('messages.edit', ['name' => 'Group Subscription']));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        GroupSubscription::where('group_subscription_id', $id)->update(['deleted_at' => Carbon::now()]);
        GroupSubscriptionTransactionHistory::where('group_subscription_id', $id)->update(['deleted_at' => Carbon::now()]);

        return redirect()->route('group.subscription.index')->with('message',  __('messages.delete', ['name' => 'Group Subscription']));
    }

    public function ajaxList(Request $request)
    {

        if ($request->ajax()) {
            $columns = ['action', 'company.company_name', 'contact_name', 'title', 'email', 'phone_no', 'number_of_subscription', 'group_subscription_status'];
            $data_spotlight_member = GroupSubscription::select('group_subscription_id', 'company.company_name', 'contact_name', 'title', 'email', 'phone_no', 'subscription_start_date', 'group_subscription_status', 'number_of_subscription', 'subscription_type');
            $data_spotlight_member->leftJoin('company', 'company.company_id', '=', 'group_subscriptions.company_name');

            $columnIndex = $request->input('order.0.column');
            $columnName = $columns[$columnIndex];
            $columnSortOrder = $request->input('order.0.dir');

            $data_spotlight_member->orderBy($columnName, $columnSortOrder);

            $data = $data_spotlight_member->get();
            return DataTables::of($data_spotlight_member)
                ->addColumn('group_subscription_status', function ($row) {
                    return $row->group_subscription_status == 1 ? "Active" : "Inactive";
                })
                ->addColumn('company_name', function ($row) {
                    return $row->company_name ? $row->company_name : '';
                })
                ->addColumn('subscription_type', function ($row) {
                    return $row->subscription_type == 1 ? 'Monthly' : 'Yearly';
                })
                ->addColumn('action', function ($row) {
                    $editUrl = route('group.subscription.edit', $row->group_subscription_id);
                    $deleteUrl = route('group.subscription.destroy', $row->group_subscription_id);
                    $ImportUser = '<button type="button" class="btn btn-light" data-toggle="modal" data-target="#importCsv" data-group_subscription_id="' . addslashes($row->group_subscription_id) . '" style="background: #e2e6ea;" title="Import User"><i class="fa fa-download" aria-hidden="true"></i></button>';
                    $userList = '<a href="' . route('app.user.index', ['group_subscription_id' =>  $row->group_subscription_id]) . '" class="btn"  style="background: #218838;" title="User List"><i class="fa fa-users" aria-hidden="true"></i></a>';
                    $subscription = '<button type="button" class="btn btn-light" data-toggle="modal" data-target="#subscription" data-group_subscription_id="' . addslashes($row->group_subscription_id) . '" style="background: #e0a800;" title="Purchased Subscription"><i class="fa fa-shopping-cart" aria-hidden="true"></i></button>';
                    $transaction_list = '<a href="' . route('group.transaction.index', ['group_subscription_id' =>  $row->group_subscription_id]) . '" class="btn"  style="background: #138496;" title="Transaction"><i class="fa fa-money" aria-hidden="true"></i></a>';

                    return '<a href="' . $editUrl . '" class="btn" style="background: #000000;color: #f0ac63 !important;" title="Edit"><i class="fa fa-pencil" aria-hidden="true"></i></a>

                    <form action="' . route('group.subscription.destroy', $row->group_subscription_id) . '" method="post" style="display:inline;" id="delete_' . $row->group_subscription_id . '">
                        <button class="btn" type="button" value="Delete" style="background: #f0ac63;color: #000000 !important;"   onclick="return deleteRecord($(this));" data-url="' . $row->group_subscription_id . '" title="Delete"/><i class="fa fa-trash" aria-hidden="true"></i></button>
                        <input type="hidden" name="_method" value="delete"/>
                        <input type="hidden" name="_token" value="' . csrf_token() . '">
                    </form> ' . $ImportUser . ' ' . $userList . ' ' . $subscription . ' ' . $transaction_list;
                })
                ->filter(function ($query) use ($request) {

                    if (isset($request->company_name)) {
                        $search = '%' . $request->company_name . '%';
                        $query->where("company.company_name", "LIKE", $search);
                    }
                    if (isset($request->contact_name)) {
                        $search = '%' . $request->contact_name . '%';
                        $query->where("contact_name", "LIKE", $search);
                    }
                    if (isset($request->email)) {
                        $search = '%' . $request->email . '%';
                        $query->where("email", "LIKE", $search);
                    }
                    if (isset($request->title)) {
                        $search = '%' . $request->title . '%';
                        $query->where("title", "LIKE", $search);
                    }
                    if (isset($request->phone_no)) {
                        $search = '%' . $request->phone_no . '%';
                        $query->where("phone_no", "LIKE", $search);
                    }
                })
                ->rawColumns(['action', 'group_subscription_status', 'subscription_type'])
                ->addIndexColumn()
                ->make(true);
        }
    }

    public function openOverlay(Request $request)
    {
        $subscripation = Subscription::pluck('subscription_name', 'subscription_id');
        $group_subscription_id = $request->input('group_subscription_id');
        return response()->json(['html' => view('group-subscription.import-user', ['group_subscription_id' => $group_subscription_id, 'subscriptions' => $subscripation])->render()]);
    }

    public function importUser(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'import_user' => 'required',
            'subscription_id' => 'required'
        ], [
            'subscription_id.required' => 'The subscription Level field is required.'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'error' => $validator->errors()
            ]);
        }

        $file = $request->file('import_user');
        $file_name = 'DataTemplate.' . $file->getClientOriginalExtension();
        $file->move(base_path('public/import-csv'), $file_name);

        $data = [
            'subscription_id' => $request->input('subscription_id'),
            'group_subscription_id' => $request->input('group_subscription_id'),
        ];

        $filePath = base_path('public/import-csv')  . '/' .  $file_name;
        $sheet_date = [];
        Excel::import(new UsersImport($data), $filePath);

        $userImport = ImportUser::select('first_name', 'last_name', 'email', 'phone_no')->where([
            'group_subscription_id' => $request->input('group_subscription_id'),
            'subscription_id' => $request->input('subscription_id')
        ])->get()->toArray();

        $appUser = AppUser::where('group_subscription_id', $request->input('group_subscription_id'))->count();
        $groupSubscription = GroupSubscription::find($request->input('group_subscription_id'));

        if (count($userImport) > ($groupSubscription['number_of_subscription'] - $appUser)) {
            ImportUser::where([
                'group_subscription_id' => $request->input('group_subscription_id'),
                'subscription_id' => $request->input('subscription_id')
            ])->delete();
            return response()->json([
                'error' => ['Number Of Subscription limit exceeded.']
            ]);
        }

        $rules = [
            '*.first_name' => 'required|string|max:255',
            '*.last_name' => 'required|string|max:255',
            '*.email' => ['required', 'email', Rule::unique('app_users')->whereNull('deleted_at')],
            '*.phone_no' => ['required', 'numeric', 'digits:10', Rule::unique('app_users')->whereNull('deleted_at')],
        ];

        $validator = Validator::make($userImport, $rules);

        if ($validator->fails()) {
            ImportUser::where([
                'group_subscription_id' => $request->input('group_subscription_id'),
                'subscription_id' => $request->input('subscription_id')
            ])->delete();
            return response()->json([
                'error' => $validator->errors()->all()
            ]);
        } else {
            dispatch(new ImportAppUser($data));
            return response()->json([
                'success' => 'Import CSV Successfully. It will take 5 minutes to process.'
            ]);
        }
    }

    public function openPurchaseOverlay(Request $request)
    {
        $subscripation = Subscription::pluck('subscription_name', 'subscription_id');
        $group_subscription_id = $request->input('group_subscription_id');
        return response()->json(['html' => view('group-subscription.purchased-subscription', ['group_subscription_id' => $group_subscription_id, 'subscriptions' => $subscripation])->render()]);
    }

    public function purchase(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'subscription_id' => 'required',
            'subscription_type' => 'required',
            'amount' => 'required',
            'subscription_start_date' => 'required'
        ], [
            'subscription_id.required' => 'Subscription Level required.',
            'subscription_type.required' => 'Subscription Type required.',
            'amount.required' => 'Amount required.',
            'subscription_start_date.required' => 'Subscription Start Date required.',
        ]);

        if ($validator->fails()) {
            return response()->json([
                'error' => $validator->errors()
            ]);
        }

        $data = $request->all();

        $groupUser = \App\Models\AppUser::where([
            'group_subscription_id' => $data['group_subscription_id'],
            'subscription_id' => $data['subscription_id']
        ])->orderBy('subscription_start_date', 'ASC')->get()->toArray();

        if (empty($groupUser)) {
            return response()->json([
                'error' => ['No App User available with this subscription.']
            ]);
        }

        $type =  $data['subscription_type'] == 1 ? '+1 months' : '+1 year';
        $startDate =  $data['subscription_start_date'] . ' ' . date('00:00:00');
        $endDate = date('Y-m-d 23:59:59', strtotime($type . ' ' . $startDate));

        GroupSubscriptionTransactionHistory::create([
            'group_subscription_id' => $data['group_subscription_id'],
            'subscription_id' => $data['subscription_id'],
            'subscription_type' => $data['subscription_type'],
            'subscription_start_date' => $startDate,
            'subscription_end_date' => $endDate,
            'amount' => $data['amount'],
            'status' => 1,
        ]);

        if (!empty($groupUser) && !empty($groupUser[0]['subscription_start_date'])) {
            $startDate = $groupUser[0]['subscription_start_date'];
        }

        if (!empty($groupUser)) {
            $groupAppUser = array_column($groupUser, 'app_user_id');
            $data = [
                'details' => [
                    'subscription_id' => $data['subscription_id'],
                    'subscription_type' => $data['subscription_type'],
                    'subscription_start_date' => $startDate,
                    'subscription_end_date' => $endDate,
                ],
                'app_user_id' => $groupAppUser,
                'subscription_id' => $data['subscription_id']
            ];
            dispatch(new AssignGroupSubscription($data));
        }

        return response()->json([
            'success' => 'Assign Subscription successfully.'
        ]);
    }

    public function subscriptionExists(Request $request){
        $data = $request->all();

        $transaction = GroupSubscriptionTransactionHistory::where([
            'group_subscription_id' => $data['group_subscription_id'],
            'subscription_id' => $data['subscription_id']
        ])
            ->where('subscription_end_date', '>=', date('Y-m-d H:i:s'))
            ->first();

        $startDate = '';
        if (!empty($transaction)) {
            $startDate = date('Y-m-d', strtotime('+1 Day ' . $transaction->subscription_end_date));
        }

        return $startDate;
    }
}
