<?php

namespace App\Http\Controllers;

use DataTables;
use Carbon\Carbon;
use App\Models\Industry;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\DB;
use App\Rules\UniqueIndustryName;
class IndustryController extends Controller
{
    public function list(Request $request)
    {
        return view('industry.indexIndustry');
    }
    public function index(Request $request, $id = 0)
    {
        if (!empty($id)) {
            $industry = Industry::where('industry_id', $id)->first();
        } else {
            $industry = '';
        }
        return view('industry.createIndustry', compact('industry'));
    }

    public function addIndustry(Request $request, $id = 0)
    {

        $data = $request->all();
        if($data['industry_id']!=null)
            {
                $this->validate($request, [
                'industry_name'=> ['required', new UniqueIndustryName($data['industry_id'])],
                ]);

            unset($data['_token']);
            Industry::where('industry_id', $data['industry_id'])->update($data);
            return redirect()->route('industry.list')->with('message', __('messages.edit', ['name' => 'Industry']));
        } else {

            $this->validate($request, [
                'industry_name'=> ['required', new UniqueIndustryName($data['industry_id'])],
            ]);

            unset($data['industry_id']);
            Industry::create($data);
            return redirect()->route('industry.list')->with('message', __('messages.add', ['name' => 'Industry']));
        }
        
    }
    public function deleteIndustry($id)
    {
        $check = DB::table('articles')->whereJsonContains('industry_id',$id)->whereNull('deleted_at')->first();
        if(!empty($check))
        {
            return redirect()->route('industry.list')->with('message_wr','This Industry can not delete because already use in articles.');
        }else{
            Industry::where('industry_id', $id)->update(['deleted_at' => Carbon::now()]);
            return redirect()->route('industry.list')->with('message', __('messages.delete', ['name' => 'Industry']));
        }
        
    }

    public function ajaxList(Request $request)
    {
        if ($request->ajax()) {
            $columns = ['action', 'industry_name', 'industry_status'];
            $columnIndex = $request->input('order.0.column');
            $columnName = $columns[$columnIndex];
            $columnSortOrder = $request->input('order.0.dir');

            $data_industry = Industry::orderBy($columnName, $columnSortOrder);
            $data_industry->get();
            return DataTables::of($data_industry)
                ->addColumn('industry_status', function ($row) {
                    if ($row->industry_status == 1) {
                        $industry_status = "Active";
                    } else {
                        $industry_status = "Inactive";
                    }
                    return $industry_status;
                })
                ->addColumn('action', function ($row) {
                    $editUrl = route('industry.form', $row->industry_id);
                    $deleteUrl = route('industry.delete', $row->industry_id);
                    return '<a href="' . $editUrl . '" class="btn" style="background: #000000;color: #f0ac63 !important;" title="Edit"><i class="fa fa-pencil" aria-hidden="true"></i></a>
                        <a href="' . $deleteUrl . '" class="btn" onclick="return confirm(\'Are you sure you want to delete this Industry?\')" style="background: #f0ac63;color: #000000 !important;" title="Delete"><i class="fa fa-trash" aria-hidden="true"></i></a>';
                })
                ->rawColumns(['industry_status', 'action'])
                ->addIndexColumn()
                ->make(true);
        }
    }
}
