<?php

//

namespace App\Http\Controllers;

use DataTables;
use Carbon\Carbon;
use App\Models\Industry;
use App\Models\Interest;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\DB;
use App\Rules\UniqueInterestName;
use App\Rules\InterestStatusChange;

class InterestController extends Controller
{
    public function list(Request $request)
    {
        return view('interest.indexInterest');
    }
    public function index(Request $request,$id=0)
    {
       if(!empty($id)){
            $interest=Interest::where('interest_id',$id)->first();
        }else{
            $interest='';
        }
        //return view('interest.createInterest');
        return view('interest.createInterest', compact('interest'));
    }
    public function addInterest(Request $request,$id=0)
    {

        $data=$request->all();
        //dd($data);
        if($data['interest_id']!=null)
            {
                $this->validate($request, [
                'interest_name'=> ['required', new UniqueInterestName($data['interest_id'])],
                // 'interest_status' => new InterestStatusChange($data['interest_id'])
                ]);

            unset($data['_token']);
            Interest::where('interest_id', $data['interest_id'])->update($data);
            return redirect()->route('interest.list')->with('message', __('messages.edit', ['name' => 'Interest']));
        }else{

            $this->validate($request, [
                'interest_name'=> ['required', new UniqueInterestName($data['interest_id'])],
            ]);

            unset($data['interest_id']);
            Interest::create($data);
            return redirect()->route('interest.list')->with('message', __('messages.add', ['name' => 'Interest']));
        }

    }
    public function deleteInterest($id)
    {
        $artical = \App\Models\Article::whereJsonContains('interest_id',$id)->count();
        if($artical > 0)
        {
            return redirect()->route('interest.list')->with('message_wr', __('messages.already_use', ['name' => 'Interest']));
        }else{
            Interest::where('interest_id', $id)->update(['deleted_at' => Carbon::now()]);
            return redirect()->route('interest.list')->with('message', __('messages.delete', ['name' => 'Interest']));
        }
        
    }

    public function ajaxList(Request $request)
    {
        if($request->ajax()){
            $columns = ['action','interest_name', 'interest_status'];
            $columnIndex = $request->input('order.0.column');
            $columnName = $columns[$columnIndex];
            $columnSortOrder = $request->input('order.0.dir');

            $data_interest = Interest::orderBy($columnName, $columnSortOrder);
            $data_interest->get();

            return DataTables::of($data_interest)
            ->addColumn('interest_status', function ($row) {
                if($row->interest_status==1)
                {
                    $interest_status="Active";
                }else{
                    $interest_status="Inactive";
                }
                return $interest_status;
            })
            ->addColumn('action', function ($row) {
                $editUrl = route('interest.form', $row->interest_id);
                $deleteUrl = route('interest.delete', $row->interest_id);
                return '<a href="'.$editUrl.'" class="btn" style="background: #000000;color: #f0ac63 !important;" title="Edit"><i class="fa fa-pencil" aria-hidden="true"></i></a>
                        <a href="'.$deleteUrl.'" class="btn" onclick="return confirm(\'Are you sure you want to delete this Interest?\')" style="background: #f0ac63;color: #000000 !important;"  title="Delete"><i class="fa fa-trash" aria-hidden="true"></i></a>';
            })
            ->rawColumns(['interest_status','action'])
            ->addIndexColumn()
            ->make(true);
        }
    }
}

