<?php

namespace App\Http\Controllers;

use DataTables;
use Carbon\Carbon;
use App\Models\Article;
use App\Models\Category;
use App\Models\Interest;
use App\Models\Publisher;
use App\Models\Subscription;
use App\Models\Company;
use App\Models\Industry;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Redirect;
use App\Rules\UniqueArticalName;
use App\Jobs\SendNotification;
use App\Models\NotificationHistory;

class PlanController extends Controller
{
    public function index()
    {
        $category = Category::status(1)->orderBy("category_name", "asc")->get();
        $interest = Interest::status(1)->orderBy("interest_name", "asc")->get();
        $subscription = Subscription::status(1)->where('subscription_status', 1)->get();
        $publisher = Publisher::status(1)->orderBy("publisher_name", "asc")->get();
        $company = Company::status(1)->orderBy("company_name", "asc")->get();
        $industry = Industry::status(1)->orderBy("industry_name", "asc")->get();

        return view('plan.create', compact('category', 'interest', 'industry', 'company', 'subscription', 'publisher'));
    }
    public function addArticle(Request $request)
    {

        $this->validate($request, [
            'article_name'=> ['required', new UniqueArticalName(0)],
        ]);

        $data = $request->all();
        if ($data['publisher'] != '') {
            $publisher_new = DB::table('publisher')->where('publisher_name', $data['publisher'])->first();
            if ($publisher_new == null) {
                $pub = [];
                $pub['publisher_name'] = $data['publisher'];
                Publisher::create($pub);
            }
        }

        $fileImage = '';
        if (isset($data['image'])) {
            if (isset($data['dbimage']) && $data['dbimage'] != '') {
                if (file_exists('image/' . $data['dbimage'])) {
                    unlink('image/' . $data['dbimage']);
                }
            }
            $file = $request->file('image');
            $destinationPath = 'image/';
            $fileImage = rand(111111, 999999) . "." . $file->getClientOriginalExtension();
            $file->move($destinationPath, $fileImage);
        }

        $thumbnailImage = '';
        if (isset($data['thumbnail_image'])) {
            $file = $request->file('thumbnail_image');
            $destinationPath = 'thumbnail_image/';
            $thumbnailImage  = rand(111111, 999999) . "." . $file->getClientOriginalExtension();
            $file->move($destinationPath, $thumbnailImage);
        }

        $resourceImage = '';
        if (isset($data['resource'])) {
            if ($resource = $request->file('resource')) {
                $destinationPath = 'resource/';
                $resourceImage = date('YmdHis') . "." . $resource->getClientOriginalExtension();
                $resource->move($destinationPath, $resourceImage);
            }
        }
        try {
            $company = isset($data['company_id']) ? json_encode($data['company_id']) : json_encode([]);
            $category = isset($data['category_id']) ? json_encode($data['category_id']) : json_encode([]);
            $industry = isset($data['industry_id']) ? json_encode($data['industry_id']) : json_encode([]);


            $detail = [
                'publisher' => $data['publisher'],
                'subscription_level' => json_encode($data['subscription_level']),
                'interest_id' => json_encode($data['interest_id']),
                'company_id' => $company,
                'category_id' => $category,
                'industry_id' => $industry,
                'image' => json_encode([$fileImage]),
                'thumbnail_image' =>  $thumbnailImage,
                'resource' => $resourceImage,
                'article_name' => $data['article_name'],
                'artical_short_desc' => $data['artical_short_desc'],
                'artical_long_desc' => $data['artical_long_desc'],
                'visiable_after_days' => $data['visiable_after_days'],
                'media_selection' => $data['media_selection'],
                'youtube_url' => isset($data['youtube_url']) ? $data['youtube_url'] : '',
                'audio' => isset($data['audio']) ? $data['audio'] : '',
                'publish_date' => isset($data['publish_date']) ? $data['publish_date'] : '',
                'search_text' => isset($data['search_text']) ? $data['search_text'] : '',
            ];
            

            $article = Article::create($detail);
            dispatch(new SendNotification(['article_id' => $article->id]));
            return redirect()->route('articles.list')->with('message', __('messages.add', ['name' => 'Article']));

        } catch (\Illuminate\Database\QueryException $ex) {
            return Redirect::back()->withError(['msg' => 'max file size exceded']);
        }


    }

    public function list(Request $request)
    {
        $categories = Category::status(1)->orderBy("category_name", "asc")->pluck('category_name', 'category_id');
        $subscriptions = Subscription::status(1)->pluck('subscription_name', 'subscription_id');
        $interests = Interest::status(1)->orderBy("interest_name", "asc")->pluck('interest_name', 'interest_id');
        $publishers = Publisher::status(1)->orderBy("publisher_name", "asc")->pluck('publisher_name');
        return view('plan.index', compact('categories', 'subscriptions', 'interests', 'publishers'));
    }

    public function handleRequest(Request $request)
    {
        $id = $request->id;
        $data_artical = Article::where('article_id', $id)->pluck('artical_long_desc')->first();

        return response()->json(['html' => $data_artical]);
    }
    public function editArticle($id)
    {
        $data_artical = Article::where('article_id', $id)->first();
        $categoryIds = json_decode($data_artical->category_id, true);
        $category = Category::where(function ($query) use ($categoryIds) {
            $query->status(1)->orWhereIn('category_id', $categoryIds);
        })->orderBy("category_name", "asc")->get();

        $interestIds = json_decode($data_artical->interest_id, true);
        $interest = Interest::where(function ($query) use ($interestIds) {
            $query->status(1)->orWhereIn('interest_id', $interestIds);
        })->orderBy("interest_name", "asc")->get();

        $subscriptionLevels = json_decode($data_artical->subscription_level, true);
        $subscription = Subscription::where(function ($query) use ($subscriptionLevels) {
            $query->status(1)->orWhereIn('subscription_id', $subscriptionLevels);
        })->where('subscription_status', 1)->get();

        $publisher = $data_artical->publisher;
        $publisher = Publisher::where(function ($query) use ($publisher) {
            $query->status(1)->orWhere('publisher_name', $publisher);
        })->orderBy("publisher_name", "asc")->get();

        $companyIds = json_decode($data_artical->company_id, true);
        $company = Company::where(function ($query) use ($companyIds) {
            $query->status(1)->orWhereIn('company_id', $companyIds);
        })->orderBy("company_name", "asc")->get();

        $industryIds = json_decode($data_artical->industry_id, true);
        $industry = Industry::where(function ($query) use ($industryIds) {
            $query->status(1)->orWhereIn('industry_id', $industryIds);
        })->orderBy("industry_name", "asc")->get();

        return view('plan.edit', compact('data_artical', 'category', 'interest', 'industry', 'company', 'subscription', 'publisher'));
    }
    public function updateArticle(Request $request)
    {
        $id = $request['article_id'];

        $this->validate($request, [
            'article_name' => ['required', new UniqueArticalName($id)],
        ]);

        $data = $request->all();
        //$data['artical_long_desc']=$description;

        $data['subscription_level'] = json_encode($data['subscription_level']);
        $data['interest_id'] = json_encode($data['interest_id']);
        if (isset($data['company_id'])) {
            $data['company_id'] = json_encode($data['company_id']);
        } else {
            $data['company_id'] = json_encode([]);
        }
        if (isset($data['category_id'])) {
            $data['category_id'] = json_encode($data['category_id']);
        } else {
            $data['category_id'] = json_encode([]);
        }
        if (isset($data['industry_id'])) {
            $data['industry_id'] = json_encode($data['industry_id']);
        } else {
            $data['industry_id'] = json_encode([]);
        }
        // $data['category_id']=json_encode($data['category_id']);
        // $data['industry_id']=json_encode($data['industry_id']);

        if (!is_null($data['remove_image'])) {
            $images = explode(',', $data['remove_image']);
            foreach ($images as  $image) {
                if (file_exists('image/' . $image)) {
                    unlink('image/' . $image);
                }
            }
            $data['image'] = '';
        }
        if (!is_null($data['remove_thumbnail_image'])) {
            if (file_exists('thumbnail_image/' . $data['remove_thumbnail_image'])) {
                unlink('thumbnail_image/' . $data['remove_thumbnail_image']);
            }
            $data['thumbnail_image'] = '';
        }
        unset($data['_token']);
        //unset($data['files']);

        if ($request->hasfile('image')) {
            if (isset($data['dbimage']) && $data['dbimage'] != '') {
                $images = explode(',', $data['dbimage']);
                foreach ($images as  $image) {
                    if (file_exists('image/' . $image)) {
                        unlink('image/' . $image);
                    }
                }
            }
            $file = $request->file('image');
            $destinationPath = 'image/';
            $fileImage = rand(111111, 999999) . "." . $file->getClientOriginalExtension();
            $file->move($destinationPath, $fileImage);
            $data['image'] = json_encode([$fileImage]);
        }

        if ($request->hasfile('thumbnail_image')) {
            if (isset($data['db_thumbnail_image']) && $data['db_thumbnail_image'] != '') {
                if (file_exists('thumbnail_image/' . $data['db_thumbnail_image'])) {
                    unlink('thumbnail_image/' . $data['db_thumbnail_image']);
                }
            }
            $file = $request->file('thumbnail_image');
            $destinationPath = 'thumbnail_image/';
            $fileImage = rand(111111, 999999) . "." . $file->getClientOriginalExtension();
            $file->move($destinationPath, $fileImage);
            $data['thumbnail_image'] = $fileImage;
        }

        if (isset($data['resource'])) {

            if ($resource = $request->file('resource')) {
                $destinationPath = 'resource/';
                $resourceImage = date('YmdHis') . "." . $resource->getClientOriginalExtension();
                $resource->move($destinationPath, $resourceImage);
                $data['resource'] = "$resourceImage";
            }
        } else {
            unset($data['resource']);
        }
        unset($data['files']);
        unset($data['remove_image']);
            unset($data['dbimage']);
        unset($data['remove_thumbnail_image']);
            unset($data['db_thumbnail_image']);
        Article::where('article_id', $id)->update($data);
        // dispatch(new SendNotification(['article_id' => $id]));

        return redirect()->route('articles.list')->with('message',  __('messages.edit', ['name' => 'Article']));
    }
    public function deleteArticle($id)
    {
        Article::where('article_id', $id)->update(['deleted_at' => Carbon::now()]);
        
        NotificationHistory::where('article_id', $id)->delete();

        return redirect()->route('articles.list')->with('message',  __('messages.delete', ['name' => 'Article']));
    }
    public function ajaxList(Request $request)
    {
        if ($request->ajax()) {
            $columns = ['action','publish_date', 'article_name', 'media_selection', 'category', 'publisher', 'subscription_level'];

            $columnIndex = $request->input('order.0.column');
            $columnName = empty($columnIndex) ? $columns[$columnIndex] : '';
            $columnSortOrder = $request->input('order.0.dir');
            $data_artical = Article::select('article_id', 'category_id', 'article_name', 'media_selection', 'publisher', 'subscription_level','publish_date');

            if (!empty($columnName) && in_array($columnName, $columns)) {
                $data_artical->orderBy($columnName, $columnSortOrder);
            } else {
                $data_artical->orderBy('article_id', $columnSortOrder);
            }

            $data_artical->get();
            return DataTables::of($data_artical)
                ->addColumn('subscription_level', function ($row) {
                    $sub = json_decode($row->subscription_level, true);
                     if(!empty($sub)){
                        $name =  \App\Models\Subscription::select(DB::raw('GROUP_CONCAT(subscription_name) AS subscription_name'))->whereIn('subscription_id', $sub)->first();
                        return $name->subscription_name;
                    }
                    return '';

                })
                ->addColumn('publisher', function ($row) {
                    return $row->publisher != 'NULL' ? $row->publisher  : '';
                })
                ->addColumn('category', function ($row) {

                    $sub = json_decode($row->category_id, true);
                    if(!empty($sub)){
                        $categoryNames = Category::whereIn('category_id', $sub)->pluck('category_name')->toArray();
                        return implode(', ', $categoryNames);
                    }

                    return '';
                })
                ->addColumn('action', function ($row) {
                    $editUrl = route('articles.edit', $row->article_id);
                    $deleteUrl = route('articles.delete', $row->article_id);

                    return '<a href="' . $editUrl . '" class="btn" style="background: #000000;color: #f0ac63 !important;" title="Edit"><i class="fa fa-pencil" aria-hidden="true"></i></a>
                        <a href="' . $deleteUrl . '" class="btn" onclick="return confirm(\'Are you sure you want to delete this Article?\')" style="background: #f0ac63;color: #000000 !important;" title="Delete"><i class="fa fa-trash" aria-hidden="true"></i></a><button type="button" class="btn btn-light" data-toggle="modal" data-target="#exampleModal" data-whatever="'.addslashes($row->article_id).'" title="Preview"><i class="fa fa-eye" aria-hidden="true"></i></button>';
                })
                ->filter(function ($query) use ($request) {
                    if (isset($request->category_id)) {
                        $query->whereRaw("JSON_CONTAINS(category_id, '\"$request->category_id\"')");
                    }

                    if (isset($request->subscription_id)) {
                        $query->whereRaw("JSON_CONTAINS(subscription_level, '\"$request->subscription_id\"')");
                    }

                    if (isset($request->interest_id)) {
                        $query->whereRaw("JSON_CONTAINS(interest_id, '\"$request->interest_id\"')");
                    }

                    if (isset($request->media_selection)) {
                        $query->where("media_selection", $request->media_selection);
                    }

                    if ($request->start_date && $request->end_date) {

                        $query->whereBetween('publish_date', [$request->start_date, $request->end_date]);
                    }

                    if (isset($request->name_search)) {

                        $search = '%' . $request->name_search . '%';
                        $query->where("article_name", "LIKE", $search);
                    }

                    if ($request->publisher) {
                        $query->where('publisher', $request->publisher);
                    }
                })
                ->rawColumns(['action', 'category'])
                ->addIndexColumn()
                ->make(true);
        }

        return view('plan.index', compact('data_artical', 'categories', 'subscriptions', 'interests', 'publishers'));
    }
}
