<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\SpotlightMember;
use DataTables;
use Carbon\Carbon;
use File;

class SpotlightMemberController extends Controller
{
    public function __construct()
    {
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index()
    {
        return view('spotlight-member.index');
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        return view('spotlight-member.form');
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
        $data = $request->all();

        $rules = [
            'spotlight_member_company_logo' => 'required|image|mimes:jpeg,jpg,png,gif',
            'spotlight_member_company_short_description' => 'required',
            'spotlight_member_company_long_description' => 'required',
            'spotlight_member_contact_email' => 'required|email',
            'spotlight_member_contact_name' => 'required',
            'spotlight_member_status' => 'required'
        ];

        $request->validate($rules);

        if (isset($data['spotlight_member_company_logo'])) {
            $file = $request->file('spotlight_member_company_logo');
            $destinationPath = 'spotlight-member/';
            $fileImage = rand(111111, 999999) . "." . $file->getClientOriginalExtension();
            $file->move($destinationPath, $fileImage);
        }

        $spotlightData = [
            "spotlight_member_company_logo" => $fileImage,
            "spotlight_member_company_short_description" => $data['spotlight_member_company_short_description'],
            "spotlight_member_company_long_description" =>  $data['spotlight_member_company_long_description'],
            "spotlight_member_contact_email" =>  $data['spotlight_member_contact_email'],
            "spotlight_member_contact_name" =>  $data['spotlight_member_contact_name'],
            "spotlight_member_status" =>  $data['spotlight_member_status']
        ];

        SpotlightMember::create($spotlightData);
        return redirect()->route('spotlight.index')->with('message', __('messages.add', ['name' => 'Spotlight Member']));

    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $spotLight = SpotlightMember::where('spotlight_member_id', $id)->first();
        return view('spotlight-member.form', ['spotLight' => $spotLight]);
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        $data = $request->all();
        $rules = [
            'spotlight_member_company_logo' => 'image|mimes:jpeg,jpg,png,gif',
            'spotlight_member_company_short_description' => 'required',
            'spotlight_member_company_long_description' => 'required',
            'spotlight_member_contact_email' => 'required|email',
            'spotlight_member_contact_name' => 'required',
            'spotlight_member_status' => 'required'
        ];
        $request->validate($rules);
        
        $spotlightMember = SpotlightMember::where('spotlight_member_id', $id)->first();

        $fileImage = $spotlightMember->spotlight_member_company_logo;
        if (isset($data['spotlight_member_company_logo'])) {
            $imagePath = public_path('spotlight-member/'.$spotlightMember->spotlight_member_company_logo);
            if(File::exists($imagePath)){
                unlink($imagePath);
            }
            $file = $request->file('spotlight_member_company_logo');
            $destinationPath = 'spotlight-member/';
            $fileImage = rand(111111, 999999) . "." . $file->getClientOriginalExtension();
            $file->move($destinationPath, $fileImage);
        }

        $spotlightData = [
            "spotlight_member_company_logo" => $fileImage,
            "spotlight_member_company_short_description" => $data['spotlight_member_company_short_description'],
            "spotlight_member_company_long_description" =>  $data['spotlight_member_company_long_description'],
            "spotlight_member_contact_email" =>  $data['spotlight_member_contact_email'],
            "spotlight_member_contact_name" =>  $data['spotlight_member_contact_name'],
            "spotlight_member_status" =>  $data['spotlight_member_status']
        ];
        SpotlightMember::where('spotlight_member_id', $spotlightMember->spotlight_member_id)->update($spotlightData);
        return redirect()->route('spotlight.index')->with('message',  __('messages.edit', ['name' => 'Spotlight Member']));
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        SpotlightMember::where('spotlight_member_id', $id)->update(['deleted_at' => Carbon::now()]);
        return redirect()->route('spotlight.index')->with('message', __('messages.delete', ['name' => 'Spotlight Member']));
    }

    public function ajaxList(Request $request)
    {

        if ($request->ajax()) {
            $columns = ['action', 'spotlight_member_contact_name', 'spotlight_member_contact_email',  'spotlight_member_company_short_description', 'spotlight_member_status', 'spotlight_member_logo'];

            $data_spotlight_member = SpotlightMember::select('spotlight_member_id', 'spotlight_member_company_logo', 'spotlight_member_company_short_description', 'spotlight_member_contact_email', 'spotlight_member_contact_name', 'spotlight_member_status');
            $columnIndex = $request->input('order.0.column');
            $columnName = $columns[$columnIndex];
            $columnSortOrder = $request->input('order.0.dir');

            $data_spotlight_member->orderBy($columnName, $columnSortOrder);
            $data_spotlight_member->get();

            return DataTables::of($data_spotlight_member)
            ->addColumn('spotlight_member_company_logo', function ($row) {
               return $imgPath = url('/') . '/spotlight-member/' .  $row->spotlight_member_company_logo;
            })
                ->addColumn('spotlight_member_status', function ($row) {
                    return $row->spotlight_member_status == 1 ? "Active" : "Inactive";
                })
                ->addColumn('action', function ($row) {
                    $editUrl = route('spotlight.edit', $row->spotlight_member_id);
                    $deleteUrl = route('spotlight.destroy', $row->spotlight_member_id);

                    return '<a href="' . $editUrl . '" class="btn" style="background: #000000;color: #f0ac63 !important;" title="Edit"><i class="fa fa-pencil" aria-hidden="true"></i></a>

                    <form action="'.route('spotlight.destroy', $row->spotlight_member_id).'" method="post" style="display:inline;" id="delete_'. $row->spotlight_member_id .'">
                        <button class="btn" type="button" value="Delete" style="background: #f0ac63;color: #000000 !important;"   onclick="return deleteRecord($(this));" data-url="'.$row->spotlight_member_id.'"  title="Delete"/><i class="fa fa-trash" aria-hidden="true"></i></button>
                        <input type="hidden" name="_method" value="delete"/>
                        <input type="hidden" name="_token" value="'. csrf_token().'">
                    </form>';
                })
                ->filter(function ($query) use ($request) {

                    if (isset($request->spotlight_member_company_short_description)) {
                        $search = '%' . $request->spotlight_member_company_short_description . '%';
                        $query->where("spotlight_member_company_short_description", "LIKE", $search);
                    }
                    if (isset($request->spotlight_member_company_long_description)) {
                        $search = '%' . $request->spotlight_member_company_long_description . '%';
                        $query->where("spotlight_member_company_long_description", "LIKE", $search);
                    }
                    if (isset($request->spotlight_member_contact_email)) {
                        $search = '%' . $request->spotlight_member_contact_email . '%';
                        $query->where("spotlight_member_contact_email", "LIKE", $search);
                    }
                    if (isset($request->spotlight_member_contact_name)) {
                        $search = '%' . $request->spotlight_member_contact_name . '%';
                        $query->where("spotlight_member_contact_name", "LIKE", $search);
                    }
                })
                ->rawColumns(['action', 'spotlight_member_status', 'spotlight_member_company_logo'])
                ->addIndexColumn()
                ->make(true);
        }
    }
}
