<?php

namespace App\Http\Controllers;

use DataTables;
use Carbon\Carbon;
use App\Models\Subscription;
use Illuminate\Http\Request;
use Illuminate\Validation\Rule;
use Illuminate\Support\Facades\DB;
use App\Rules\SubscriptionStatusChange;
use App\Rules\UniqueSubscriotionName;
class SubscriptionController extends Controller
{
    public function list(Request $request)
    {
        return view('subscription.indexSubscription');
    }


    public function userList(){
        $data = Subscription::all();
        return view('subscription.userSubscription',compact('data'));
    }

    public function index(Request $request, $id = 0)
    {
        if (!empty($id)) {
            $subscription = Subscription::where('subscription_id', $id)->first();
        } else {
            $subscription = '';
        }
        return view('subscription.createSubscription', compact('subscription'));
    }

    public function addSubscription(Request $request, $id = 0)
    {

        $data = $request->all();

        $subscriptionData = [
            'subscription_name' => $data['subscription_name'],
            'subscription_content' => $data['subscription_content'],
            'subscription_price_per_month' => $data['subscription_price_per_month'],
            'subscription_price_per_year' => $data['subscription_price_per_year'],
            'subscription_status' => $data['subscription_status'],
        ];
        if ($data['subscription_id'] != null) {

                $this->validate($request, [
                'subscription_name'=> ['required', new UniqueSubscriotionName($data['subscription_id'])],
                ]);

            unset($data['_token']);
            Subscription::where('subscription_id', $data['subscription_id'])->update($subscriptionData);
            return redirect()->route('subscription.list')->with('message', __('messages.edit', ['name' => 'Subscription']));
        } else {
            $this->validate($request, [
                'subscription_name'=> ['required', new UniqueSubscriotionName(0)],
            ]);
            unset($data['subscription_id']);
            Subscription::create($subscriptionData);
            return redirect()->route('subscription.list')->with('message', __('messages.add', ['name' => 'Subscription']));
        }
    }
    public function deleteSubscription($id)
    {
        $check = DB::table('articles')->whereJsonContains('subscription_level',$id)->whereNull('deleted_at')->first();
        if(!empty($check))
        {
            return redirect()->route('subscription.list')->with('message_wr','This subscription can not delete because already use in articles.');
        }else{
            Subscription::where('subscription_id', $id)->update(['deleted_at' => Carbon::now()]);
            return redirect()->route('subscription.list')->with('message', __('messages.delete', ['name' => 'Subscription']));
        }
       
    }

    public function ajaxList(Request $request)
    {

        if ($request->ajax()) {

            $columns = ['action','subscription_level', 'subscription_name', 'subscription_price_per_month', 'subscription_price_per_year', 'subscription_status'];
            $columnIndex = $request->input('order.0.column');
            $columnName = $columns[$columnIndex];
            $columnSortOrder = $request->input('order.0.dir');

            $data_subscription = Subscription::orderBy($columnName, $columnSortOrder);

            $data_subscription->get();

            return DataTables::of($data_subscription)
                ->addColumn('subscription_status', function ($row) {
                    if ($row->subscription_status == 1) {
                        $subscription_status = "Active";
                    } else {
                        $subscription_status = "Inactive";
                    }
                    return $subscription_status;
                })
                ->addColumn('subscription_price_per_month', function ($row) {
                    
                    $subscription_price_per_month = "$".number_format($row->subscription_price_per_month,2);
                   
                    return $subscription_price_per_month;
                })
                ->addColumn('subscription_price_per_year', function ($row) {
                    
                    $subscription_price_per_year = "$".number_format($row->subscription_price_per_year,2);
                   
                    return $subscription_price_per_year;
                })
                ->addColumn('action', function ($row) {
                    $editUrl = route('subscription.form', $row->subscription_id);
                    $deleteUrl = '';
                    // $deleteUrl =  '<a href="' . route('subscription.delete', $row->subscription_id) . '" class="btn" onclick="return confirm(\'Are you sure you want to delete this Subscription?\')" style="background: #f0ac63;color: #000000 !important;">Delete</a>';
                    return '<a href="' . $editUrl . '" class="btn" style="background: #000000;color: #f0ac63 !important;"  title="Edit"><i class="fa fa-pencil" aria-hidden="true"></i></a>' . $deleteUrl;
                }) ->filter(function ($query) use ($request) {


                    if (isset($request->subscription_name)) {

                        $search = '%' . $request->subscription_name . '%';
                        $query->where("subscription_name", "LIKE", $search);
                    }

                    if ($request->subscription_id ) {
                        $query->where('subscription_id', $request->subscription_id );
                    }

                })
                ->rawColumns(['subscription_status', 'action', 'spotlight_member_company_short_description'])
                ->addIndexColumn()
                ->make(true);
        }
    }


    public function addUserSubscription($subscription_id){
        $subscription = Subscription::where('subscription_id',$subscription_id)->first();
        return view('subscription.user-pay',compact('subscription'));
    }
}
