<?php

namespace App\Http\Controllers;

use DataTables;
use App\Models\PaymentLog;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Auth;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\Rule;
use net\authorize\api\contract\v1 as AnetAPI;
use net\authorize\api\controller as AnetController;
use Mail;
use DateTime;

class TransactionController extends Controller
{
    public function __construct()
    {
    }
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function index(Request $request)
    {
        $full_name = '';
        $app_user_id = $request->query('app_user_id') ? $request->query('app_user_id') : '';
        if($app_user_id != ''){
            $app_user = \App\Models\AppUser::where('app_user_id', $app_user_id)->first();
            $full_name = $app_user->first_name . ' '. $app_user->last_name;
        }
        return view('transaction.index', ['app_user_id' => $app_user_id, 'full_name' => $full_name]);
    }

    public function appUserTracation(){

        $columns = ['payment_log_id','first_name', 'title', 'phone_no', 'type','authorize_net_id', 'payment_logs.created_at', 'subscription.subscription_name', 'payment_logs.subscription_type', 'payment_logs.date', 'status'];
            $transaction  = PaymentLog::select('first_name', 'last_name', 'title', 'phone_no','authorize_net_id', DB::raw('payment_logs.created_at'), DB::raw('payment_logs.subscription_id'),  DB::raw('payment_logs.subscription_type'), DB::raw('payment_logs.date'),'status', DB::raw('subscription.subscription_name'), 'message', 'type');
            $transaction->leftJoin('app_users', 'app_users.app_user_id', '=', 'payment_logs.app_user_id');
            $transaction->leftJoin('subscription', 'subscription.subscription_id', '=', 'payment_logs.subscription_id');

            $transaction->where('payment_logs.app_user_id', Auth::guard('app_user')->user()->app_user_id);
            
            $transaction->orderBy('payment_log_id', 'DESC');
            $data = $transaction->get();
            return view('transaction.user-index', compact('data'));
    }

    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
    }

    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */
    public function store(Request $request)
    {
    }

    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function show($id)
    {
        //
    }

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
      //
    }

    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request, $id)
    {
        //
    }

    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        //
    }

    public function ajaxList(Request $request)
    {

        if ($request->ajax()) {
            $columns = ['first_name', 'title', 'phone_no', 'type','authorize_net_id', 'payment_logs.created_at', 'subscription.subscription_name', 'payment_logs.subscription_type', 'payment_logs.date', 'status'];
            $transaction  = PaymentLog::select('first_name', 'last_name', 'title', 'phone_no','authorize_net_id', DB::raw('payment_logs.created_at'), DB::raw('payment_logs.subscription_id'),  DB::raw('payment_logs.subscription_type'), DB::raw('payment_logs.date'),'status', DB::raw('subscription.subscription_name'), 'message', 'type');
            $transaction->leftJoin('app_users', 'app_users.app_user_id', '=', 'payment_logs.app_user_id');
            $transaction->leftJoin('subscription', 'subscription.subscription_id', '=', 'payment_logs.subscription_id');

            $columnIndex = $request->input('order.0.column');
            $columnName = $columns[$columnIndex];
            $columnSortOrder = $request->input('order.0.dir');

            if($request->app_user_id != 0){
                $transaction->where('payment_logs.app_user_id', $request->app_user_id);
            }

            $transaction->orderBy($columnName, $columnSortOrder);

            $data = $transaction->get();
            return DataTables::of($transaction)
                ->addColumn('full_name', function ($row) {
                    return $row->first_name . ' '. $row->last_name;
                })
                ->addColumn('subscription_type', function ($row) {
                    return $row->subscription_type == 1 ? 'Monthly' : 'Yearly';
                })
                ->addColumn('type', function ($row) {
                    return $row->type == 1 ? 'Subscription' : 'Transaction';
                })
                ->addColumn('date', function ($row) {
                    return !is_null($row->date) ?  date('Y-m-d H:i:s', strtotime($row->date)) : '-';
                })
                ->filter(function ($query) use ($request) {

                    if (isset($request->name)) {
                        $query->where(DB::raw('CONCAT(first_name, " ", last_name)'), $request->name);
                    }
                    if (isset($request->phone_no)) {
                        $search = '%' . $request->phone_no . '%';
                        $query->where("phone_no", "LIKE", $search);
                    }
                    if (isset($request->authorize_net_id)) {
                        $search = '%' . $request->authorize_net_id . '%';
                        $query->where("authorize_net_id", "LIKE", $search);
                    }
                    if (isset($request->subscription_type)) {
                        $query->where("payment_logs.subscription_type", $request->subscription_type);
                    }
                    if (isset($request->status)) {
                        $query->where("payment_logs.status", $request->status);
                    }
                    if (isset($request->type)) {
                        $query->where("payment_logs.type", $request->type);
                    }
                    if (isset($request->created_at)) {
                        $query->whereRaw("DATE(payment_logs.created_at) = ?", [$request->created_at]);
                    }
                })
                ->rawColumns(['status', 'full_name', 'subscription_type'])
                ->make(true);
        }
    }

    public function userList(Request $request)
    {
        $param = $request->input('param');
        $search = '%' . $param . '%';
        return \App\Models\AppUser::select(DB::raw('CONCAT(first_name, " ", last_name)  AS full_name'), 'app_user_id')->where(DB::raw('CONCAT(first_name, " ", last_name)'), "LIKE", $search)
        ->pluck('full_name', 'app_user_id');
    }


    public function payment(Request $request)
    {

        $request->validate([
            'card_number' => 'required|numeric',
            'exp_date' => 'required',
            'code' => 'required|numeric',
            'amount' => 'required',
            'subscription_id' => 'required',
            'subscription_type' => 'required|in:1,2',
        ]);

        $data = $request->all();
        $cardNumber = $data['card_number'];
        $expDate = $data['exp_date'];
        $code = $data['code'];
        $amount = $data['amount'];
        $subscriptionId = $data['subscription_id'];
        $subscriptionType = $data['subscription_type'];
        $userId = \Auth::guard('app_user')->user()->app_user_id;
        $type =  $subscriptionType == 1 ? '+1 months' : '+1 year';
        $startDate = date('Y-m-d H:i:s');
        $endDate = date('Y-m-d H:i:s', strtotime($type));

        $uniqueId = uniqid();
        $prefix = "INV";
        $invoiceNumber = $prefix . "-" . $uniqueId;

        /* Create a merchantAuthenticationType object with authentication details retrieved from the constants file */
        $merchantAuthentication = new AnetAPI\MerchantAuthenticationType();
        $merchantAuthentication->setName(env('MERCHANT_LOGIN_ID'));
        $merchantAuthentication->setTransactionKey(env('MERCHANT_TRANSACTION_KEY'));

        // dd(env('MERCHANT_LOGIN_ID'), env('MERCHANT_TRANSACTION_KEY'),env('PAYMENT_ENVIRONMENT'));
        // Set the transaction's refId
        $refId = 'ref' . time();
        $intervalLength = 30;

        // Subscription Type Info
        $subscription = new AnetAPI\ARBSubscriptionType();
        $subscription->setName("Subscription");

        $interval = new AnetAPI\PaymentScheduleType\IntervalAType();
        $interval->setLength($intervalLength);
        $interval->setUnit("days");

        $paymentSchedule = new AnetAPI\PaymentScheduleType();
        $paymentSchedule->setInterval($interval);
        $paymentSchedule->setStartDate(new DateTime());
        $paymentSchedule->setTotalOccurrences(9999);

        $subscription->setPaymentSchedule($paymentSchedule);
        $subscription->setAmount($amount);

        $creditCard = new AnetAPI\CreditCardType();
        $creditCard->setCardNumber($cardNumber);
        $creditCard->setExpirationDate($expDate);

        $payment = new AnetAPI\PaymentType();
        $payment->setCreditCard($creditCard);
        $subscription->setPayment($payment);


        $order = new AnetAPI\OrderType();
        $order->setInvoiceNumber($invoiceNumber);
        $order->setDescription("Subscription");
        $subscription->setOrder($order);

        $billTo = new AnetAPI\NameAndAddressType();
        $billTo->setFirstName(\Auth::guard('app_user')->user()->first_name);
        $billTo->setLastName(\Auth::guard('app_user')->user()->last_name);

        $subscription->setBillTo($billTo);

        $request = new AnetAPI\ARBCreateSubscriptionRequest();
        $request->setmerchantAuthentication($merchantAuthentication);
        $request->setRefId($refId);
        $request->setSubscription($subscription);
        $controller = new AnetController\ARBCreateSubscriptionController($request);

        // Create the controller and get the response
        if (env('PAYMENT_ENVIRONMENT') == 'SANDBOX') {
           
            $response = $controller->executeWithApiResponse(\net\authorize\api\constants\ANetEnvironment::SANDBOX);

        } else {
            $response = $controller->executeWithApiResponse(\net\authorize\api\constants\ANetEnvironment::PRODUCTION);
        }

        if (($response != null) && ($response->getMessages()->getResultCode() == "Ok")) {

            $update = [
                'subscription_id' => $subscriptionId,
                'subscription_type' => $subscriptionType,
                'authorize_subscription_id' => $response->getSubscriptionId(),
                'authorize_subscription_status' => 1,
                'subscription_start_date' => date('Y-m-d H:i:s'),
                'subscription_end_date' => date('Y-m-d H:i:s', strtotime('+100 years')),
            ];

            \App\Models\AppUser::where('app_user_id', $userId)->update($update);

            $subscription = \App\Models\Subscription::where('subscription_id', $subscriptionId)->pluck('subscription_name');

            $paymentLog = [
                [
                    'app_user_id' => \Auth::guard('app_user')->user()->app_user_id,
                    'authorize_net_id' => $response->getSubscriptionId(),
                    'type' => 1,
                    'message_code' => 'net.authorize.customer.subscription.created',
                    'message' => 'Subscription Successfully',
                    'amount' => $amount,
                    'subscription_id' => $subscriptionId,
                    'subscription_type' => $subscriptionType,
                    'date' => date('Y-m-d H:i:s'),
                    'invoice_number' => $invoiceNumber,
                    'status' => 'Create'
                ], [
                    'app_user_id' => \Auth::guard('app_user')->user()->app_user_id,
                    'authorize_net_id' => '',
                    'type' => 2,
                    'message_code' => '',
                    'message' => 'Transaction Successfully',
                    'amount' => $amount,
                    'subscription_id' => $subscriptionId,
                    'subscription_type' => $subscriptionType,
                    'date' => date('Y-m-d H:i:s'),
                    'invoice_number' => $invoiceNumber,
                    'status' => 'Success'
                ]

            ];

            \App\Models\PaymentLog::insert($paymentLog);
            $msg = $response->getMessages()->getMessage()[0]->getText();

            $data = [
                'start' => $startDate,
                'name' =>  \Auth::guard('app_user')->user()->first_name . ' ' . \Auth::guard('app_user')->user()->last_name,
                'subscripation' => $subscription[0],
                'email' => 'dipak.imphatic@gmail.com'
            ];

            try {
                Mail::send('mails.subscription-successfully', ['data' => $data], function ($message) use ($data) {
                    $message->to($data['email'], $data['name'])->subject('Luxury Council –Subscription Purchased successfully');
                });
            } catch (\Exception $e) {
                Log::error('This is an error message.' . $e);
            }


            $detail = [
                'amount' => $amount,
                'status' => true,
                'transaction_id' => $response->getSubscriptionId(),
                'auth_id' => '',
                'message_code' => '',
                'message' =>   $response->getMessages()->getMessage()[0]->getText(),
                'created_at' => date('Y-m-d H:i:s'),
                'app_user_id' => $userId,
                'subscription_id' => $subscriptionId,
                'subscription_type' => $subscriptionType,
            ];

            // $response = [
            //     'data' => $detail,
            //     'status' => true,
            //     'message' => $msg
            // ];
            return back()->with('success', $msg);
        } else {
            return back()->with('error', $response->getMessages()->getMessage()[0]->getText());
            // $response = [
            //     'data' => (object) [],
            //     'status' => false,
            //     'message' =>   $response->getMessages()->getMessage()[0]->getText()
            // ];
        }

        
    }
}
