<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\User;
use Carbon\Carbon;
use DataTables;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Hash;
use Illuminate\Validation\Rule;
class UserController extends Controller
{
    public function list(Request $request)
    {

        return view('users.indexUser');
    }

    public function index(Request $request, $id = 0)
    {
        if (!empty($id)) {
            $user = User::where('id', $id)->first();
        } else {
            $user = '';
        }
        return view('users.createUser', compact('user'));
    }

    public function addUser(Request $request, $id = 0)
    {

        $data = $request->all();

        $rules = [
            'first_name' => 'required',
            'last_name' => 'required',
            'phone_number' => ['required', 'numeric', 'digits:10', Rule::unique('users')->whereNull('deleted_at')->whereNot('id', $data['id'])],
            'email' => ['required', 'email' , Rule::unique('users')->whereNull('deleted_at')->whereNot('id', $data['id'])],
        ];

        if($data['id'] == null){
            $rules['password'] = 'min:6|required_with:c_password|same:c_password';
            $rules['c_password'] = 'required';
        }
        $request->validate($rules, [
            'password.same' => 'The password and confirm password must match.',
            'c_password.required' => 'The confirm password field is required..'
        ]);

        if ($data['id'] != null) {
            unset($data['_token']);
            unset($data['c_password']);

            if($data['password'] != ''){
                $data['password'] = Hash::make($data['password']);
            }else {
                unset($data['password']);
            }
            User::where('id', $data['id'])->update($data);
            return redirect()->route('users.list')->with('message', __('messages.edit', ['name' => 'User']));
        } else {

            unset($data['id']);
            unset($data['c_password']);
            $data['password'] = Hash::make($data['password']);
            User::create($data);

            return redirect()->route('users.list')->with('message',__('messages.add', ['name' => 'User']));
        }
    }
    public function deleteUser($id)
    {
        User::where('id', $id)->update(['deleted_at' => Carbon::now()]);
        return redirect()->route('users.list')->with('message', __('messages.delete', ['name' => 'User']));
    }

    public function ajaxList(Request $request)
    {
        if ($request->ajax()) {

            $columns = ['action','first_name', 'email', 'phone_number', 'user_role', 'user_status'];
            $columnIndex = $request->input('order.0.column');
            $columnName = $columns[$columnIndex];
            $columnSortOrder = $request->input('order.0.dir');

            $data_user = User::query()->orderBy($columnName, $columnSortOrder);
            $data_user->get();

            return DataTables::of($data_user)
                ->addColumn('user_role', function ($row) {
                    return ($row->user_role == 1) ? 'Admin' : 'Assistant';
                })
                ->addColumn('user_status', function ($row) {
                    if ($row->user_status == 1) {
                        $user_status = "Active";
                    } else {
                        $user_status = "Inactive";
                    }
                    return $user_status;
                })->addColumn('full_name', function ($row) {
                    return $row->first_name . ' ' . $row->last_name;
                })
                ->addColumn('action', function ($row) {
                    $edit = '<a href="' . route('users.form', $row->id) . '" class="btn" style="background: #000000;color: #f0ac63 !important;"  title="Edit"><i class="fa fa-pencil" aria-hidden="true"></i></a>';
                    $delete = '<a href="' . route('users.delete', $row->id) . '" class="btn" onclick="return confirm(\'Are you sure you want to delete this User?\')" style="background: #f0ac63;color: #000000 !important;"  title="Delete"><i class="fa fa-trash" aria-hidden="true"></i></a>';

                    return $edit . ($row->user_role != 1 ? ' ' . $delete : '');
                })->filter(function ($query) use ($request) {
                    if (isset($request->full_name)) {
                        $search = '%' . $request->full_name . '%';
                        $query->whereRaw("CONCAT(first_name, ' ', last_name) LIKE  '$search'");
                    }
                    if ($request->phone_number) {
                        $search = '%' . $request->phone_number . '%';
                        $query->where("phone_number", "LIKE", $search);
                    }
                    if ($request->email) {
                        $search = '%' . $request->email . '%';
                        $query->where("email", "LIKE", $search);
                    }
                    if ($request->user_role) {
                        $query->where("user_role",  $request->user_role);
                    }
                })
                ->rawColumns(['user_status', 'action', 'full_name', 'user_role'])
                ->addIndexColumn()
                ->make(true);
        }
    }
}
