<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Log;
use net\authorize\api\contract\v1 as AnetAPI;
use net\authorize\api\controller as AnetController;

class WebHookController extends Controller
{

    public function subscription(Request $request)
    {
        $payload = file_get_contents("php://input");
        Log::info('Subscription Webhook', [$payload]);
        $webhook = json_decode($payload, true);

        $subscriptionId = $webhook['payload']['id'];
        $amount =  $webhook['payload']['amount'];
        $appUser = \App\Models\AppUser::select('app_user_id', 'subscription_id', 'subscription_type')->where('authorize_subscription_id', $subscriptionId)->first();

        $message = $status = '';
        if ($webhook['eventType'] == 'net.authorize.customer.subscription.cancelled') {
            $status = 'Cancelled';
            $message = 'Subscription Cancelled';
        } else if ($webhook['eventType'] == 'net.authorize.customer.subscription.suspended') {
            $status = 'Suspended';
            $message = 'Subscription Suspended';
        } else if ($webhook['eventType'] == 'net.authorize.customer.subscription.expired') {
            $status = 'Expired';
            $message = 'Subscription Expired';
        } else {
            $status = 'Terminated';
            $message = 'Subscription Terminated';
        }

        $paymentLog = [
            'app_user_id' => $appUser->app_user_id,
            'authorize_net_id' => $subscriptionId,
            'type' => 1,
            'message_code' => $webhook['eventType'],
            'message' => $message,
            'amount' => $amount,
            'subscription_id' => $appUser->subscription_id,
            'subscription_type' => $appUser->subscription_type,
            'date' => date('Y-m-d H:i:s'),
            'authorize_net_response' => $payload,
            'status' => $status
        ];

        \App\Models\AppUser::where('authorize_subscription_id', $subscriptionId)->update(
            [
                'authorize_subscription_status' => 0,
                'subscription_end_date' => date('Y-m-d H:i:s'),
                'authorize_subscription_id' => ''
            ]
        );

        \App\Models\PaymentLog::insert($paymentLog);

        return true;
    }

    public function transaction(Request $request)
    {
        $payload = file_get_contents("php://input");
        Log::info('Transaction Webhook', [$payload]);
        $response = json_decode($payload, true);

        $transactionId = $response['payload']['id'];
        $amount =  $response['payload']['authAmount'];
        $invoice =  isset($response['payload']['invoiceNumber']) ? $response['payload']['invoiceNumber'] : '';
        $eventType =  $response['eventType'];
        

        Log::info('$invoice Webhook', [$invoice]);
        
        
        if (isset($response['payload']['invoiceNumber']) && \App\Models\PaymentLog::where('invoice_number', $response['payload']['invoiceNumber'])->where('type', 2)->exists() == true) {

            $paymentLog = [
                'authorize_net_id' => $transactionId,
                'message_code' => $eventType,
                'authorize_net_response' => $response,
            ];
            Log::info('$paymentLog Webhook', [$paymentLog]);

            \App\Models\PaymentLog::where('invoice_number', $response['payload']['invoiceNumber'])->where('type', 2)->update($paymentLog);
        } else {

            /* Create a merchantAuthenticationType object with authentication details retrieved from the constants file */
            $merchantAuthentication = new AnetAPI\MerchantAuthenticationType();
            $merchantAuthentication->setName(env('MERCHANT_LOGIN_ID'));
            $merchantAuthentication->setTransactionKey(env('MERCHANT_TRANSACTION_KEY'));

            $refId = 'ref' . time();

            $request = new AnetAPI\GetTransactionDetailsRequest();
            $request->setMerchantAuthentication($merchantAuthentication);
            $request->setTransId($transactionId);

            $controller = new AnetController\GetTransactionDetailsController($request);

            // Create the controller and get the response
            if (env('PAYMENT_ENVIRONMENT') == 'SANDBOX') {
                $response = $controller->executeWithApiResponse(\net\authorize\api\constants\ANetEnvironment::SANDBOX);
            } else {
                $response = $controller->executeWithApiResponse(\net\authorize\api\constants\ANetEnvironment::PRODUCTION);
            }
            Log::info('$response', [$response]);

            $subscriptionId =  $response->getTransaction()->getSubscription()->getId();
            Log::info('$subscriptionId', [$subscriptionId]);
            $appUser = \App\Models\AppUser::select('app_user_id', 'subscription_id', 'subscription_type')
                ->where('authorize_subscription_id', $subscriptionId)->first();
            Log::info('$appUser', [$appUser]);
            $message = 'Transaction Successfully';
            $status = 'Success';

            $paymentLog = [
                'app_user_id' => $appUser->app_user_id,
                'authorize_net_id' => $transactionId,
                'type' => 2,
                'message_code' => $eventType,
                'message' => $message,
                'amount' => $amount,
                'subscription_id' => $appUser->subscription_id,
                'subscription_type' => $appUser->subscription_type,
                'date' => date('Y-m-d H:i:s'),
                'authorize_net_response' => $response,
                'status' => $status,
                'invoice_number' => $invoice
            ];
            Log::info('$paymentLog', [$paymentLog]);

            $insert = \App\Models\PaymentLog::insert($paymentLog);
            Log::info('$insert', [$insert]);
        }
        return true;
    }
}
