<?php

namespace App\Imports;

use App\Models\ImportUser;
use Maatwebsite\Excel\Concerns\ToModel;
use Maatwebsite\Excel\Concerns\Importable;
use Maatwebsite\Excel\Concerns\WithHeadingRow;
use Maatwebsite\Excel\Concerns\WithValidation;
use Illuminate\Validation\Rule;
class UsersImport implements ToModel, WithHeadingRow, WithValidation
{
    use Importable;

    private $data;

    public function __construct(array $data = [])
    {
        $this->data = $data;
    }


    public function rules(): array
    {
        return [
            'first_name' => 'required|string|max:255',
            'last_name' => 'required|string|max:255',
            'email' => ['required', 'email', Rule::unique('app_users')->whereNull('deleted_at'), Rule::unique('import_user')] ,
            'phone_no' => ['required', 'numeric', 'digits:10' , Rule::unique('app_users')->whereNull('deleted_at'), Rule::unique('import_user')],
            'mailing_address' => 'required',
        ];
    }

    /**
    * @param array $row
    *
    * @return \Illuminate\Database\Eloquent\Model|null
    */
    public function model(array $row)
    {

        return new ImportUser([
            "first_name" => $row['first_name'],
            "last_name" => $row['last_name'],
            "email" => $row['email'],
            "phone_no" => $row['phone_no'],
            "mailing_address" => $row['mailing_address'],
            "group_subscription_id" => $this->data['group_subscription_id'], // User Type User
            "subscription_id" => $this->data['subscription_id']
        ]);
    }
}
