<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldBeUnique;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use App\Mail\SendDigestMail;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;

class DigestMail implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    protected $details;

    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($details)
    {
        $this->details = $details;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $appUsers = $this->details['app_user'];
        foreach ($appUsers as $appUser) {
            $articles = $this->details['article'];
            if ($appUser['subscription_id'] == 6) {
                $articles = $articles->where('company_id', $appUser['company']);
            }

            if(!empty($articles)){
                $data = [
                    'article' => $articles,
                    'app_user' => $appUser
                ];
                Mail::to($appUser['email'])->send(new SendDigestMail($data));
            }
        }
    }
}
