<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldBeUnique;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Mail;
use App\Models\ImportUser;
use App\Models\GroupSubscription;
use Illuminate\Support\Facades\Hash;
use App\Models\AppUser;
use Illuminate\Support\Facades\Log;


class ImportAppUser implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;


    public $details;
    /**
     * Create a new job instance.
     *
     * @return void
     */
    public function __construct($details)
    {
        $this->details = $details;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $userImport = ImportUser::select('import_user_id', 'first_name', 'last_name', 'email', 'phone_no', 'mailing_address')->get()->toArray();
        $groupSubscription = GroupSubscription::select()->where('group_subscription_id', $this->details['group_subscription_id'])->first()->toArray();

        foreach ($userImport as $key => $val) {

            $password =  $this->randomPassword();

            $appUser = [
                'group_subscription_id' => $this->details['group_subscription_id'],
                'first_name' => $val['first_name'],
                'last_name' => $val['last_name'],
                'email' => $val['email'],
                'password' =>  Hash::make($password),
                'company' => $groupSubscription['company_name'],
                'title' => $groupSubscription['title'],
                'country_code' => 'US',
                'phone_no' => $val['phone_no'],
                'mailing_address' => $val['mailing_address'],
                'subscription_id' => $this->details['subscription_id'],
                'app_user_status' => $groupSubscription['group_subscription_status']
            ];

            $transaction = \App\Models\GroupSubscriptionTransactionHistory::where([
                'group_subscription_id' => $this->details['group_subscription_id'],
                'subscription_id' => $this->details['subscription_id']
            ])->where('subscription_end_date', '>=', date('Y-m-d H:i:s'));

            if ($transaction->count() > 0) {
                $data = $transaction->first();
                $groupUser = \App\Models\AppUser::where([
                    'group_subscription_id' => $this->details['group_subscription_id'],
                    'subscription_id' => $this->details['subscription_id']
                ])->orderBy('subscription_start_date', 'ASC');

                $startDate = date('Y-m-d',strtotime($data->subscription_start_date)) . ' ' . date('00:00:00');

                if ($groupUser->count() > 0 && !empty($groupUser->subscription_start_date)) {
                    $startDate = $groupUser->subscription_start_date;
                }
                $appUser['subscription_start_date'] = $startDate;
                $appUser['subscription_end_date'] = $data->subscription_end_date;
                $appUser['subscription_type'] = $data->subscription_type;
            }

            AppUser::create($appUser);

            $mail_data = [
                'first_name' => $val['first_name'],
                'last_name' => $val['last_name'],
                'password' =>  $password,
                'email' => $appUser['email']
            ];

            ImportUser::where(['import_user_id' => $val['import_user_id']])->delete();

            Mail::send('mails.credential', ['data' => $mail_data], function ($message) use ($mail_data) {
                $message->to($mail_data['email'], $mail_data['first_name'] . ' ' . $mail_data['last_name'])->subject('Luxury Council - App Credential');
            });
        }
    }

    public function randomPassword($length = 8)
    {
        $uppercase = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $lowercase = 'abcdefghijklmnopqrstuvwxyz';
        $numbers = '0123456789';
        $specialChars = '!@#$%^&*()-_+=<>?';

        // Concatenate all character sets
        $allChars = $uppercase . $lowercase . $numbers . $specialChars;

        // Ensure at least one character from each set
        $password = $uppercase[rand(0, 25)] .
                    $lowercase[rand(0, 25)] .
                    $numbers[rand(0, 9)] .
                    $specialChars[rand(0, strlen($specialChars) - 1)];

        // Fill the rest of the password with random characters
        $password .= substr(str_shuffle($allChars), 0, $length - 4);

        // Shuffle the password to make it more random
        return $password = str_shuffle($password);

    }
}
