<?php

namespace App\Jobs;

use Illuminate\Bus\Queueable;
use Illuminate\Contracts\Queue\ShouldBeUnique;
use Illuminate\Contracts\Queue\ShouldQueue;
use Illuminate\Foundation\Bus\Dispatchable;
use Illuminate\Queue\InteractsWithQueue;
use Illuminate\Queue\SerializesModels;
use Illuminate\Support\Facades\Log;

class SendNotification implements ShouldQueue
{
    use Dispatchable, InteractsWithQueue, Queueable, SerializesModels;

    /**
     * Create a new job instance.
     *
     * @return void
     */

    public $details;

    public function __construct($details)
    {
        $this->details = $details;
    }

    /**
     * Execute the job.
     *
     * @return void
     */
    public function handle()
    {
        $articleId = $this->details['article_id'];

        $levels = ['1', '2', '3', '4', '5', '6'];
        foreach ($levels as $level) {

            if (!is_null($level)) {
                $visibleLevel = config('constant.visible_article')[$level];
            } else {
                $visibleLevel = config('constant.non_subscribe_user');
            }

            $articleIds = [];

            foreach ($visibleLevel as $key => $val) {
                $query = \App\Models\Article::select('article_name', 'artical_short_desc', 'company_id', 'article_id');

                if ($val == 2) {
                    $nonVisibleArticle = $this->getArticleAfterApplyVisiableDate();
                    if (!empty($nonVisibleArticle)) {
                        $query->whereNotIn('article_id', $nonVisibleArticle);
                    }
                }

                $query->where(function ($q) use ($val) {
                    $q->orWhereRaw("JSON_CONTAINS(subscription_level, '\"$val\"')");
                });

                $article =  $query->orderby('publish_date', 'DESC')->get();

                if (!empty($article->pluck('article_id')->toArray())) {
                    $articleIds = array_merge($articleIds, $article->pluck('article_id')->toArray());
                }
            }

            if (in_array($articleId, $articleIds)) {
                $article =  \App\Models\Article::select('article_name', 'artical_short_desc', 'company_id')->where('article_id', $articleId)->first();

                if (!empty($article)) {
                    $query = \App\Models\AppUser::where('subscription_id', $level)->pluck('app_user_id');
                    if ($level == 6) {
                        $query->whereIn('company_id', json_decode($article->company_id));
                    }
                    $appUsers =  $query->toArray();

                    if (!empty($appUsers)) {

                        // Insert Article Notification History
                        foreach($appUsers as $appUserId){
                            \App\Models\NotificationHistory::create(['article_id'=>$articleId, 'app_user_id' => $appUserId]);
                        }

                        $device = \App\Models\DeviceToken::whereIn('app_user_id', $appUsers)->pluck('device_token')->all();

                        $images = [];
                        if (!empty($article->image)) {

                            foreach (json_decode($article->image, true) as $image) {
                                if ($image != '') {
                                    $images[]['imageurl'] = url('/') . '/image/' . $image;
                                }
                            }
                        }

                        $thumbnail_image = '';
                        if (!empty($article->thumbnail_image)) {
                            $thumbnail_image = url('/') . '/thumbnail_image/' . $article->thumbnail_image;
                        }

                        if (!empty($device)) {
                            $data = [
                                "registration_ids" => $device,
                                "notification" => [
                                    "title" => $article->article_name,
                                    "body" => $article->artical_short_desc,
                                ],
                                "data" => [
                                    "article_id" => $articleId,
                                    "title" => $article->article_name,
                                    "body" => $article->artical_short_desc,
                                    "image" => $images,
                                    "thumbnail_image" => $thumbnail_image,
                                ]
                            ];

                            $dataString = json_encode($data);

                            $headers = [
                                'Authorization: key=' . env('FCM_SERVER_KEY'),
                                'Content-Type: application/json',
                            ];

                            $ch = curl_init();

                            curl_setopt($ch, CURLOPT_URL, 'https://fcm.googleapis.com/fcm/send');
                            curl_setopt($ch, CURLOPT_POST, true);
                            curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
                            curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
                            curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
                            curl_setopt($ch, CURLOPT_POSTFIELDS, $dataString);

                            $response = curl_exec($ch);
                        }
                    }
                }
            }


        }
    }

    public function getArticleAfterApplyVisiableDate()
    {
        return \App\Models\Article::select('article_id')->whereRaw("JSON_CONTAINS(subscription_level, '\"2\"')")
            ->whereRaw("JSON_LENGTH(subscription_level) = 1")
            ->whereRaw("DATE(now()) < DATE_ADD(publish_date, INTERVAL visiable_after_days DAY)")
            ->whereNotNull('visiable_after_days')->get()->pluck('article_id');
    }
}
