<?php

namespace App\Models;

use Illuminate\Database\Eloquent\Factories\HasFactory;
use Illuminate\Database\Eloquent\Model;
use Illuminate\Database\Eloquent\Relations\HasOne;
use Illuminate\Database\Eloquent\SoftDeletes;
use Illuminate\Foundation\Auth\User as Authenticatable;
use Tymon\JWTAuth\Contracts\JWTSubject;
use Carbon\Carbon;

class AppUser extends Authenticatable implements JWTSubject
{
    use HasFactory, SoftDeletes;
    protected $table = 'app_users';
    protected $primaryKey = "app_user_id";

    protected $appends  = ['subscription_expired', 'profile_img_url'];
    
    protected $fillable = [
        'first_name',
        'last_name',
        'email',
        'password',
        'company',
        'title',
        'country_code',
        'phone_no',
        'mailing_address',
        'assistant_name',
        'assistant_email',
        'assistant_country_code',
        'assistant_phone_no',
        'subscription_id',
        'subscription_type',
        'app_user_status',
        'group_subscription_id',
        'subscription_start_date',
        'subscription_end_date',
        'birthday',
        'personal_interests',
        'business_interests',
        'spouse_name',
        'spouse_birthday',
        'authorize_subscription_status',
        'profile_img'
    ];

    protected $hidden = [
        'created_at',
        'updated_at',
        'password',
    ];

    
    public function getJWTIdentifier()
    {
        return $this->getKey();
    }

    public function getJWTCustomClaims()
    {
        return [];
    }

    public function getSubscriptionExpiredAttribute($value)
    {
        return isset($this->attributes['subscription_end_date']) ? ((strtotime($this->attributes['subscription_end_date']) <= strtotime(date('Y-m-d H:i:s'))) ? 1 : ($this->attributes['authorize_subscription_status']  == 0 ?  1  : 0 ) ) : 1;
    }

    public function children() {
        return $this->hasMany(\App\Models\AppUserChildDetail::class, 'app_user_id', 'app_user_id')->select('app_user_child_detail_id', 'app_user_id', 'name', 'birthday');
    }

    public function getSubscriptionEndDateAttribute($value)
    {
        if($this->subscription_start_date != ''){
            $start_date = Carbon::parse($this->subscription_start_date); //     Convert the start date to a Carbon object

            if ($this->subscription_type == 1) {
                // Add 1 year to the start date
                $start_date->setYear(date('Y'));
                $end_date = $start_date->addMonth();
            } elseif ($this->subscription_type == 2) {
                // Add 2 years to the start date
                $start_date->setYear(date('Y'));
                $end_date = $start_date->addYear(1);
            }
    
            $end_date->setTime(23, 59, 59);
            // Format the end date as 'Y-m-d' (e.g., 2023-10-25)
            return $end_date->format('Y-m-d H:i:s');
        } 
        return null;
        
    }
    
        public function getProfileImgUrlAttribute($value)
    {
        return ($this->profile_img != '') ? url('/profile_image/'. $this->profile_img)  : '';
    }
}
